#include        <time.h>       /* local time                        */
#include        <math.h>       /* pow                               */
#include        "shaza.h"      /* for shaza structs and defines etc */
#include        "command.h"    /* for shaza command line        etc */

/*************************************************************************************/
double dataBlockRecaps(nloci,nBlocks,anim,nrec,Etotal,rBlocks,model,ci)  /* from real data */
/* coded by: Michael Macbeth       revision - 5/10/2009     */
int nloci;           /* number of loci                      */
int nBlocks;         /* number of populations in data       */
int anim;            /* number of genotype samples in data  */
int nrec;            /* good pairs having LLR < z_value     */
double Etotal;       /* effective size of all pooled data using z_value */
int *rBlocks;        /* blocks with recaps having LLR < z_value */
int model;
int ci;              /* convergence iteration */
/*************************************************************************************/
{
   int i,j;
   int blocksWithRecaps=0;        /* blocks found with recaptures */
   int blocksWithGoodRecaps=0; /* blocks found with recaptures with Match>Shadows */
   int blist[BLOCK];   /* block combination of match */
   int rlist[BLOCK];   /* number of recaps in block combination */
   int ia,ib,iaa,ibb;
   double K;
   double pctR,pctR1,pctR2,pctR12;    /* percentage corrected recapture */
   double corR1,corR2,corR12;         /* corrected recapture */
   double PIR,PIR1,PIR2,PIR12;        /* Probability of individual recapture */
   double E=0.0,E1, E2,E12;               /* Effective sample size */
   int N1,N2;                /* block size */
   double N12;               /* block size */
   double alpha;             /* estimated number of false positives in block */
   char pct='%';
   double total_corr_recaps=0.0;  /* total corrected recaps */
   double kij;
   double totalNotEnabled;
   time_t time_now;

   time(&time_now); /* get time in seconds */

   if( (fp8 = fopen(TEMP_FILE, "w" )) == NULL )
   {
        fprintf( stderr, "dataBlockRecaps(): Cannot open <%s> for writing\n",TEMP_FILE);
        exit( -1 );
   }
   else
   {
        fprintf(fp8,"%s\n",asctime(localtime(&time_now)));
        fprintf(fp8,"Executable file: %s\n",VERSION);
        if(i_option[0]) fprintf(fp8,"Input data file: %s\n",i_value[0]);
        if(i_option[1]) fprintf(fp8,"Input data file: %s\n",i_value[1]);
                        fprintf(fp8,"Output     file: %s\n\n",o_value[3]);
   }

   totalNotEnabled=anim*(anim-1)/2.0-Etotal*(Etotal-1)/2.0;

   if(nrec == 0)
   {
       fprintf(fp8,"\n\n____________________________________________________________________________________________________\n");
       fprintf(fp8,"Convergence iteration %d with %5.3f false positives\n",ci+1,alphaCut[z_value]);
       fprintf(fp8,"No recaptures found: \n");
   }
   else
   {
      fprintf(fp8,"\n\n\n\n\n\n\n\n________________________________________________________\n");
                      fprintf(fp8,"Convergence iteration %d with %5.3f false positives\n",ci+1,alphaCut[z_value]);
                      fprintf(fp8,"________________________________________________________\n\n");
      
      if(nBlocks == 1)
      {
        fprintf(fp8,"\n\n\nTable 1  Within population recapture estimate from reference data\n");
        fprintf(fp8,"____________________________________________________________________________________________________\n");
        fprintf(fp8,"  Population    Population                 Effective       False       Matches  Recaptures__________\n");
        fprintf(fp8,"  number        size                       sample size     positives   found    Corrected   (%c) \n",pct);
        fprintf(fp8,"  n1            s1                          E               V          M        R_hat        P        \n");
        fprintf(fp8,"____________________________________________________________________________________________________\n");
      }
      else
      {
        fprintf(fp8,"\n\n\nTable 1  Within and between population recapture estimates from data\n");
        fprintf(fp8,"____________________________________________________________________________________________________\n");
        fprintf(fp8,"  Population    Population    Effective    Effective       False       Matches  Recaptures__________\n");
        fprintf(fp8,"  number        size          block size   sample size     positives   found    Corrected   (%c) \n",pct);
        fprintf(fp8,"  n1   n2       s1   s2        b            e               V          M        R_hat        P  \n");
        fprintf(fp8,"____________________________________________________________________________________________________\n");
      }

      /* initialise */
      for(i=0; i<BLOCK; i++) rlist[i]=0;

      /* for all matches found in data collate blocks with recaps */
      get_blist_rlist(nrec,blist,rlist,&blocksWithRecaps,0);

      /* calculate within block recap estimates */
      for(j=0; j<blocksWithRecaps; j++)
      {
         ibb=blist[j]/BLOCKFIT;
         iaa=blist[j]-ibb*BLOCKFIT;
         ia=iaa-1;
         ib=ibb-1;
         if(iaa == ibb)
         {
              N1=(nBlock[iaa]-nBlock[ia]);
              if(nBlocks == 1)
              {
                E=d_Ex[z_value];   /* already estimated if only one block */      
                kij=E*(E-1)/2.0;
              }
              else             E=effectiveBlockSize(model,nloci,anim,ia,ib,0,&kij); /* Find E within block */


              /* Partition alpha (false positives) determined from all data into an estimate of alpha within current block */
              /* This assumes false positives equally likely per not-enabled comparison in different spatial/temporal groups */
              if(totalNotEnabled > 1)
                alpha=alphaCut[z_value]*(N1*(N1-1)/2-kij)/totalNotEnabled;
              else
              /* If all pairwise comparisons enabled then partition alpha on number of pairwise comparisons in each block */
                alpha=alphaCut[z_value]*(N1*(N1-1)/2)/(anim*(anim-1)/2);


              if(rlist[j] > alpha)   /* if recaptures greater than number of false positives */
              {
                 /* use binomial probability to determine PIR of individual match */
                 PIR=1.0-exp(log(1-(rlist[j]-alpha)/E)/E);
                 corR12=N1*(1.0-pow((1.0-PIR),N1));
                 pctR=100.0*corR12/(N1-corR12);   /* N1 corrected for recaps found */


                 /* record the blocks of recaps for simulation of data */
                 simBlock[blocksWithGoodRecaps].ia=ia;
                 simBlock[blocksWithGoodRecaps].ib=ib;
                 simBlock[blocksWithGoodRecaps].up=0;
                 simBlock[blocksWithGoodRecaps++].corR=corR12;
                 total_corr_recaps=total_corr_recaps+corR12;

                 if(nBlocks == 1)
                 fprintf(fp8,"%4d        %6d                      %9.2f      %9.3f  %6d    %9.2f   %9.2f  \n",
                                    iaa,
                                    N1,
                                    E,
                                    alpha,
                                    rlist[j],
                                    corR12,
                                    pctR
                                    );
                 else
                 fprintf(fp8,"%4d : %-4d %6d : %-6d %9.2f   %9.2f      %9.3f  %6d    %9.2f   %9.2f  \n",
                                    iaa,
                                    ibb,
                                    N1,
                                    N1,
                                    (N1*1.0),
                                    E,
                                    alpha,
                                    rlist[j],
                                    corR12,
                                    pctR
                                    );
              }
              else
              {
                 fprintf(fp8,"%4d : %-4d %6d : %-6d %9.2f   %9.2f      %9.3f  %6d    NOTE: Shadows (S) were greater than matches found (M)  \n",
                                    iaa,
                                    ibb,
                                    N1,
                                    N1,
                                    (N1*1.0),
                                    E,
                                    alpha,
                                    rlist[j]
                                    );
              }

         }
         else  /* off-diagonal block */
         {

              N1=nBlock[iaa]-nBlock[ia];
              N2=nBlock[ibb]-nBlock[ib];
              N12=sqrt(N1*N2);    /* effective block size */

              E12=effectiveBlockSize(model,nloci,anim,ia,ib,0,&kij);

              K=E12*E12;
              E1=sqrt(K*N1/N2);
              E2=sqrt(K*N2/N1);

              /* stochastic simulation may lead to some variables to cause log problems */
              /* slight adjustments below help make this program more robust */
              if(E1 > N1) E1=N1;
              if(E2 > N2) E2=N2;
              if(E12 > N12) E12=N12;

              if(E1 < rlist[j] ) {  fprintf(fp8,"dataBlockRecaps: Warning E1=%f < rlist[%d]=%d\n",E1,j,rlist[j]); E1=rlist[j]; }
              if(E2 < rlist[j] ) {  fprintf(fp8,"dataBlockRecaps: Warning E2=%f < rlist[%d]=%d\n",E2,j,rlist[j]); E2=rlist[j]; }
              if(E12 < rlist[j] ) {  fprintf(fp8,"dataBlockRecaps: Warning E12=%f < rlist[%d]=%d\n",E12,j,rlist[j]); E12=rlist[j]; }

              if(totalNotEnabled > 1)
              /* Partition alpha (false positives) determined from all data into an estimate of alpha within current block */
              /* This assumes false positives equally likely per non-enabled comparison in different spatial/temporal groups */
                alpha=alphaCut[z_value]*(N1*N2-kij)/(totalNotEnabled);
              else
              /* If all pairwise comparisons enabled then partition alpha on number of pairwise comparisons in each block */
                alpha=alphaCut[z_value]*(N1*N2/2)/(anim*(anim-1)/2);

              #if DEBUG==1
              if( (1-(rlist[j]-alpha)/E1) < SMALL)
              {
                 fprintf(stdout,"Cannot get log of %f in dataBlockRecaps() j=%d rlist[j]=%d alpha=%f  E1=%f\n",
                    (1-(rlist[j]-alpha)/E1),j,rlist[j],alpha,E1 );
                 fprintf(fp7,"Cannot get log of %f in dataBlockRecaps() j=%d rlist[j]=%d alpha=%f  E1=%f\n",
                    (1-(rlist[j]-alpha)/E1),j,rlist[j],alpha,E1 );
                 exit(1);
              }
              if( (1-(rlist[j]-alpha)/E2) < SMALL)
              {
                 fprintf(stdout,"Cannot get log of %f in dataBlockRecaps() j=%d rlist[j]=%d alpha=%f  E2=%f\n",
                    (1-(rlist[j]-alpha)/E2),j,rlist[j],alpha,E2 );
                 fprintf(fp7,"Cannot get log of %f in dataBlockRecaps() j=%d rlist[j]=%d alpha=%f  E2=%f\n",
                    (1-(rlist[j]-alpha)/E2),j,rlist[j],alpha,E2 );
                 exit(1);
              }
              if( (1-(rlist[j]-alpha)/E12) < SMALL)
              {
                 fprintf(stdout,"Cannot get log of %f in dataBlockRecaps() j=%d rlist[j]=%d alpha=%f  E12=%f\n",
                    (1-(rlist[j]-alpha)/E12),j,rlist[j],alpha,E12 );
                 fprintf(fp7,"Cannot get log of %f in dataBlockRecaps() j=%d rlist[j]=%d alpha=%f  E12=%f\n",
                    (1-(rlist[j]-alpha)/E12),j,rlist[j],alpha,E12 );
                 exit(1);
              }
              #endif

              if(rlist[j] > alpha)
              {

                 PIR1=1.0-exp(log(1-(rlist[j]-alpha)/E1)/E1);
                 PIR2=1.0-exp(log(1-(rlist[j]-alpha)/E2)/E2);
                 PIR12=1.0-exp(log(1-(rlist[j]-alpha)/E12)/E12);
                 corR1=N1*(1.0-pow((1.0-PIR1),N1));
                 corR2=N2*(1.0-pow((1.0-PIR2),N2));
                 corR12=N12*(1.0-pow((1.0-PIR12),N12));
                 pctR1=100.0*corR1/(N1);
                 pctR2=100.0*corR2/(N2);
                 pctR12=100.0*corR12/(N12);


                 /* record the blocks of recaps for simulation of data */
                 simBlock[blocksWithGoodRecaps].ia=ia;
                 simBlock[blocksWithGoodRecaps].ib=ib;
                 simBlock[blocksWithGoodRecaps].up=0;
                 simBlock[blocksWithGoodRecaps++].corR=corR12;
                 total_corr_recaps=total_corr_recaps+corR12;

                 fprintf(fp8,"%4d : %-4d %6d : %-6d %9.2f   %9.2f      %9.3f  %6d    %9.2f   %9.2f \n",
                                    iaa,
                                    ibb,
                                    N1,
                                    N2,
                                    N12,
                                    E12,
                                    alpha,
                                    rlist[j],      /* this includes false positives */
                                    corR12,
                                    pctR12);
              }
              else
              {
                 fprintf(fp8,"%4d : %-4d %6d : %-6d %9.2f   %9.2f      %9.3f  %6d    NOTE: matches found (M) were greater than the number of false positives (S)   \n",
                                    iaa,
                                    ibb,
                                    N1,
                                    N1,
                                    (N1*1.0),
                                    E,
                                    alpha,
                                    rlist[j]
                                    );
              }
         }
      }
   }
   fprintf(fp8,"____________________________________________________________________________________________________\n");
   *rBlocks=blocksWithGoodRecaps;
   fclose(fp8);
   return(total_corr_recaps);
}
