#include  <math.h>  /* pow sqrt                          */
#include  <stdio.h> /* for debug */
#include  "shaza.h" /* for shaza structs and defines etc */
#include  "command.h" /* for shaza structs and defines etc */

/*************************************************************************************/
double effectiveBlockSize(model,nloci,anim,ia,ib,isim,kij)
/* coded by: Michael Macbeth       revision - 24/06/2011     */
/* returns effective size using Likelihood cut-off value */
/*************************************************************************************/
int model;
int nloci;         /* number of loci in data */
int ia;            /* block a */
int ib;            /* block b */
int isim;          /* =1 simulated data; =0 real data */
double *kij;       /* enabled comparisons in i j th block */
{
   int i,j;
   unsigned int bit;         /* common loci between samples i and j */
   double xii;               /* LLR between sample i and i */
   double xjj;               /* LLR between sample j and j*/
   double x;                 /* LLR between sample i and j */

   double K=0.0;             /* number of comparisons yielding < defined number of false positives */
                             /* made double as could be larger than sizeof(int)? */
   int a_start;              /* start position of block a */
   int a_end;                /* end   position of block a */
   int b_start;              /* start position of block b */
   int b_end;                /* end   position of block b */

   a_start=nBlock[ia];
   a_end=nBlock[ia+1];

   if(ia == ib) /* diagonal block */
   {
      for(i=a_start; i < a_end; i++)       /* lower triangular animal matrix  (vertical)  */
      {
         for(j=a_start; j < i;  j++)       /* lower triangular animal matrix  (horizontal) */
         {
           bit=rec[i].mbit&rec[j].mbit;
           if(bit)                   /* if common bits (loci) between samples i and j */
           {
             likeme2(model,i,j,nloci,anim,bit,isim,&x,&xjj,&xii);

             /* add numbers in SFZ (this probably includes most matches) */
             if(xii > LLRcut[z_value]) K=K+0.5;
             if(xjj > LLRcut[z_value]) K=K+0.5;
           }
         }
      }
      *kij=K;
      return(sqrt(8.0*K+1.0)/2.0+0.5);
   }
   else  /* non-diagonal block */
   {
      b_start=nBlock[ib];
      b_end=nBlock[ib+1];

      for(i=a_start; i < a_end; i++)      /* lower animal matrix   (vertical)  */
      {
         for(j=b_start; j < b_end; j++)   /* lower animal matrix  (horizontal) */
         {
           bit=rec[i].mbit&rec[j].mbit;
           if(bit)                                   /* if common bits between samples i and j */
           {              
             likeme2(model,i,j,nloci,anim,bit,isim,&x,&xjj,&xii);

             /* add numbers in SFZ (this probably includes most matches) */
             if(xii > LLRcut[z_value]) K=K+0.5;
             if(xjj > LLRcut[z_value]) K=K+0.5;
           }
         }
      }
      *kij=K;
      return(sqrt(K));
   }
}
