#include <math.h>
#include "shaza.h"
#include "command.h"
/*************************************************************************************/
void find_like_recap_data(model,nloci,anim,nblocks,loop,isim,totalPairs,effeciveSize)
/* coded by: Michael Macbeth       revision - 7/10/2009     */
/* calculate matching allele scores between animals */
/*************************************************************************************/
int model;
int nloci;         /* number of loci in data */
int anim;          /* samples read in data */
int nblocks;       /* number of blocks in data */
int loop;          /* current simulation iteration */
int isim;          /* =1 for simulation else =0 */
int *totalPairs;   /* number of paired matches */
double *effeciveSize; /* effective size of all data using z_value */
{
   int i,j,k;
   unsigned int bit;
   double x;
   static int done=0;

   double xii;
   double xjj;
   double PIR;           /* probability of individual recapture */
   double Rc;            /* corrected recaptures */
   double Rp;            /* percentage corrected recaptures */
   double Rr;            /* recaptures = total pairs - false positives */
   double EE;            /* effective size of data */

   int ng[RANK_SFZ];     /* count number of matches found i.e. those LLR < LLR_S */
   int position1;
   int notgotit;
   int ia,ib;            /* block numbers from 0.. */
   int nrec=0;           /* cumulative number of match records */
   double var1;          /* variance of LLR given parent j */
   double var2;          /* variance of LLR given parent i */
   double count_pairs[RANK_SFZ];
   double K;             /* number of comparisons with < given number of false positives */

   /* for missmatch analysis */
   int count,different;
   int ncount[NOL],ndifferent[NOL];
   
   /* if reference data */
   if(!done)
   for(k=0; k<(nloci+1); k++)
   {
     if(loop == 0)
     {
        Ex[0][k]=0.0;     Ex[1][k]=0.0;
        R[0][k]=0.0;      R[1][k]=0.0;
        Rcor[0][k]=0.0;   Rcor[1][k]=0.0;  Rcor[2][k]=0.0;
        pctR[0][k]=0.0;   pctR[1][k]=0.0;
        ngood[0][k]=0.0;  ngood[1][k]=0.0;
        PIRs[0][k]=0.0;   PIRs[1][k]=0.0;
     }
     ng[k]=0;
     ncount[k]=0;
     ndifferent[k]=0;
   }

   /* initialise     */
   for(k=0; k <RANK_SFZ; k++)
   {
     if(loop == 0)
     {
        d_nrec=0;
        d_Ex[k]=0.0;
        d_R[k]=0.0;
        d_Rcor[k]=0.0;
        d_pctR[k]=0.0;  
        d_ngood[k]=0.0; 
        d_PIRs[k]=0.0; 
     }
     ng[k]=0;
     count_pairs[k]=0.0;
   }


   for(i=0; i < anim; i++)       /* lower triangular animal matrix   (verticle)  */
   {
      ia=inblock(i,nblocks);
      for(j=0; j < i;  j++)       /* lower triangular animal matrix  (horizontal) */
      {
        bit=rec[i].mbit&rec[j].mbit;
        if(bit)                   /* if common bits between samples i and j */
        {

          /* record missmatch profile */
          if(!done)
          {
            loci_missmatch(nloci,i,j,&count,&different);
            ncount[count]++;
            ndifferent[different]++;
          }
         
          ib=inblock(j,nblocks);

          likeme2(model,i,j,nloci,anim,bit,isim,&x,&xjj,&xii,&var1,&var2);

          position1=binarySearch(LLRcut, 0, RANK_SFZ-1, x);

          if( position1 > -1)
          {
             notgotit=1;   /* have not recorded this match so far */
             for(k=position1; k<RANK_SFZ; k++)  /* all matches with k_th+ false positive rank are good matches */
             {
                ng[k]++;   /* count number of good matches at incremental z_values */
                if(nrec < PAIRS)  /* Only store first PAIRS pairs found */
                if(k < MAX_RANK)  /* Store all matches up to MAX_RANK for listing (eg matches plus those that missed out) */
                {                 /* the cumulative number of false positives will approximately equal MAX_RANK */
                                  /* Only store match for the first k-th listing */
                   if(notgotit)
                   {
                      store_match_pairs(nrec,i,j,ia,ib,x,bit,isim);
                      nrec++;
                      if(nrec == PAIRS)
                      {
                        fprintf(fp7,"Warning only first %d pairs recorded.\nTo read more pairs change PAIRS and recompile.",PAIRS);
                        fprintf(stdout,"Warning only first %d pairs recorded.\nTo read more pairs change PAIRS and recompile.",PAIRS);
                      }
                   }
                   notgotit=0;
                }
             }
          }

          /* add numbers in SFZ: this should also include all good matches with no allele errors  */
          for(k=0; k <RANK_SFZ; k++)
          {
            if(xii > LLRcut[k]) count_pairs[k]++;
            if(xjj > LLRcut[k]) count_pairs[k]++;
          }
        }
      }
   }


   /* pooled data analysis from all blocks */
   for(k=0; k<RANK_SFZ; k++)
   {

     /* R - number of recaptures found less false positives */
     Rr=ng[k]-alphaCut[k];
     if(Rr < 0.0) Rr=0.0;

     /* Effective sample size */
     /* find effective size based on count_pairs[k] */
     K=count_pairs[k];
     EE=sqrt(4.0*K+1.0)/2.0+0.5;


     x=1.0-Rr/EE;
     if(x <= 0.0) PIR=0.0;
     else         PIR=1.0-exp(log(x)/EE);

     Rc=anim*(1.0-pow((1.0-PIR),anim));

     Rp=100*Rc/(anim-Rc);  /* % recapture */

#ifdef jk
     ngood[0][k]=ngood[0][k]+ng[k];       /* sigma X     */   /* ngood - number of recaptures found (includes possible shadows) */
     ngood[1][k]=ngood[1][k]+ng[k]*ng[k]; /* sigma X*X   */
     R[0][k]= R[0][k]+Rr;                 /* sigma X   */
     R[1][k]= R[1][k]+Rr*Rr;              /* sigma X*X */
     Ex[0][k]=Ex[0][k]+EE;                /* sigma X   */
     Ex[1][k]=Ex[1][k]+EE*EE;             /* sigma X*X */
     PIRs[0][k]=PIRs[0][k]+PIR;           /* sigma X   */
     PIRs[1][k]=PIRs[1][k]+PIR*PIR;       /* sigma X*X */
     Rcor[0][k]=Rcor[0][k]+Rc;            /* sigma X   */
     Rcor[1][k]=Rcor[1][k]+Rc*Rc;         /* sigma X*X */
     Rcor[2][k]=Rcor[2][k]+Rc*Rc*Rc;      /* sigma X*X*X */
     pctR[0][k]=pctR[0][k]+Rp;            /* sigma X   */
     pctR[1][k]=pctR[1][k]+Rp*Rp;         /* sigma X*X */

     if(k == z_value)   *effeciveSize=EE; /* effective size for this single run within loops */
#endif

     d_ngood[k]=d_ngood[k]+ng[k];       /* sigma X   */   /* ngood - number of recaptures found (includes possible false positives) */
     d_R[k]= d_R[k]+Rr;                 /* sigma X   */
     d_Ex[k]=d_Ex[k]+EE;                /* sigma X   */
     d_PIRs[k]=d_PIRs[k]+PIR;           /* sigma X   */
     d_Rcor[k]=d_Rcor[k]+Rc;            /* sigma X   */
     d_pctR[k]=d_pctR[k]+Rp;            /* sigma X   */
d_nrec=nrec;

//fprintf(stdout,"isim=%d k=%d EE=%f K=%f Rc=%f Rr=%f \n",isim,k,EE,K,Rc,Rr);
   }

*effeciveSize=d_Ex[z_value];
   *totalPairs=nrec;

   /* report missmatch profile */
   if(!done)
   {
     done=1;
     fprintf(fp2,"\n\n Pairwise missmatch analysis of loci\n");
     fprintf(fp2,"____________________________________\n");
     fprintf(fp2," Number      Comparable    Mismatch\n");
     fprintf(fp2,"____________________________________\n");
     for(k=0; k< (nloci+1); k++ )
     {
       fprintf(fp2," %3d     %12d %12d\n",k,ncount[k],ndifferent[k]);
     }
     fprintf(fp2,"____________________________________\n");
   }
   
}

