#include <math.h>
#include "shaza.h"
#include "command.h"


/*************************************************************************************/
int get_recaps_from_sim(model,nloci,anim,nBlocks,rBlocks,total_corr_recaps,compG,ci)
/* coded by: Michael Macbeth       revision - 27/10/2009     */
/* finds recapture estimates with s.e. */
/*************************************************************************************/
int model;
int nloci;    /* number of loci */
int anim;     /* nummber of genotype samples */
int nBlocks;  /* total number of blocks of data */
int rBlocks;  /* number of blocks in reference data with a recap */
double total_corr_recaps;  /* total corrected recaps across all blocks */
int compG;    /* composite genotypes (used when one block only) */
int ci;       /* convergence iteration */
{
   int i,k,loop;
   int iperm;  /* calculated but not needed */
   int ia,ib;
   unsigned int rec_mbit_L[ANIM];
   double Etotal;
   int totalPairs;
   int z001;
   double sd,se, seMin=999999.0;
   double mls_cutoff;
   int kMin=0;
   int best_z;
   int new_z;
   double sum_vw=0.0;


   /* initialise     */
   mls_cutoff=1/n_value; /* detectable limit to determine false positives */
   /* Rse[] stores std_deviation of each z_value run */
   /* set to undefined value of -1.0  */
   for(k=0; k <RANK_SFZ; k++) if(!ci) Rse[0][k]=-1.0;

   fprintf(stdout,"\n");
   fprintf(fp7,"\n");
   if((rBlocks > 0)||(nBlocks == 1))       /* if there were blocks with recaptures OR only one block */
   {
      for(loop=0; loop <s_value; loop++)  /* Likelihood Shadow Simulation Runs*/
      {

#if VERBOSE
         if(c_value == 1)
         {
            fprintf(stdout,"\n          run=%2d/%d",loop+1,s_value); fflush(stdout);
            fprintf(fp7   ,"\n          run=%2d/%d",loop+1,s_value);
         }
         else
         {
            fprintf(stdout,"\n          Convergence_iteration=%d      run=%2d/%d",ci+1,loop+1,s_value); fflush(stdout);
            fprintf(fp7   ,"\n          Convergence_iteration=%d      run=%2d/%d",ci+1,loop+1,s_value);
            
         }
#endif
         if(b_value == 1)   /* Bootstrap loci combinations from all data */
         {
           if(!loop) /* copy rec[].mbit once from original data */
           for(i=0; i< anim; i++) rec_mbit_L[i]=rec[i].mbit;   /* Store missing loci combinations of data */
           /* randomly define missing loci combinations for each animal */

           /* create new missing loci combinations */
           bitperm(0,0,1,' ');   /* initialise rec[].mbit */
           get_mbit3(anim,&iperm,anim,rec_mbit_L);  /* randomly bootstrap missing loci combinations for each animal */

         }
         else
         if(b_value == 2)  /* Bootstrap loci combinations from within temporal/spatial groups */
         {
           if(!loop) /* copy rec[].mbit once from original data */
           for(i=0; i< anim; i++) rec_mbit_L[i]=rec[i].mbit;   /* Store missing loci combinations of data */
           /* randomly define missing loci combinations for each animal */

           /* create new missing loci combinations */
           bitperm(0,0,1,' ');   /* initialise rec[].mbit */
           get_mbit4(anim,&iperm,anim,rec_mbit_L);  /* randomly bootstrap missing loci combinations for each animal */
         }
         /* else use combinations from reference data in same order i.e. rec[i].mbit */

         /* fills animal_sim[n][i][j] adds recaps pairs   */
         if(sim_individual_match_genotypes(model,nloci,anim,rBlocks,compG,total_corr_recaps) < 0) loop=s_value;

         
         /* calculates total recaps assuming data in one big block */
         /* Table 2 SHAZA_pop.txt data generated in find_like_recap if nBlock=1 */
         find_like_recap(model,nloci,anim,nBlocks,loop,1,&totalPairs,&Etotal);  /* stored info across all z_value's */
                       /* Etotal is the Effective size from current loop */
         /* calculates recaps in individual blocks before summing to find total recaps */

         if(nBlocks >1)        /* create Table 2 SHAZA_pop.txt by simulating within each block */
         if(totalPairs > 0) 
         {
             simBlockRecaps(model,nloci,anim,loop,totalPairs,nBlocks,rBlocks,Etotal); /* stores info for -z value only */
         }

      } /* end s.e. loop */


      /* get sum_vw across all blocks (variance attributed to simulating fractional recaptures with whole numbers)  */
      for(ia=0; ia<nBlocks; ia++)
      for(ib=0; ib<nBlocks; ib++)
      {
        if(sum_rlist[ia][ib] > SMALL)
        {
          if(nBlocks == 1) se=get_se(ia,ib,rBlocks,Rcor[0][z_value],Rcor[1][z_value],&sum_vw);
          else             se=get_se(ia,ib,rBlocks,sd_corR12[ia][ib][0],sd_corR12[ia][ib][1],&sum_vw);
        }
      }

      /* Find z_value with corrected recaptures with smallest se */
      /* with Matches found > Shadows and store in kMin          */
      /* NOTE: se = sd when there is only one reference data     */
      kMin=0;
      for(k=0; k<(RANK_SFZ); k++)
      {
        sd=std_dev(s_value,Rcor[0][k],Rcor[1][k]);
        se=sqrt(sd*sd-sum_vw);
        if(se < seMin) {  seMin=se; kMin=k; }

        /* store mean and std error for this z_value run */
        if(k == z_value)
        {
          Rse[0][z_value]=se;
          Rse[1][z_value]=Rcor[0][k]/s_value;
          Rse[2][z_value]=total_corr_recaps;
        }
      }


      for(i=0; i< anim; i++) rec[i].mbit=rec_mbit_L[i];   /* reset original data missing loci combinations */

      fprintf(fp3,"\n\n\n_____________________________________________________\n");
      fprintf(fp3,"Convergence iteration %d with %5.3f false positives\n",ci+1,alphaCut[z_value]);
      fprintf(fp3,"_____________________________________________________\n\n");
      fprintf(fp3,"________________________________________________________________________________________________\n");

      if((R_option)&&(USE_R_VALUE))
      fprintf(fp3,"Table 2  Recaptures found when simulating %-3.2f pairwise comparisons with given false positives. \n",(double)R_value);
      else
      fprintf(fp3,"Table 2  Recaptures found when simulating %-3.2f pairwise comparisons with given fale positives. \n",total_corr_recaps);

      if(nBlocks > 1)
      fprintf(fp3,"         For recapture estimates within populations see: %s\n",o_value[3]);
      fprintf(fp3,"         Standard error (s.e.) estimated from %d simulation runs\n",s_value);
      fprintf(fp3,"         '*' indicates the estimate with minimum variance\n");
      fprintf(fp3,"         '#' indicates the z value used to estimate recaptures from reference data\n");
      fprintf(fp3,"________________________________________________________________________________________________\n");


      fprintf(fp3,"\n\n");
      fprintf(fp3,"  z    False     Matches     Recaps.  False      Effective      Corrected       Percentage\n");
      fprintf(fp3,"       positives                      negatives  size           recaptures      recaptures\n");
      fprintf(fp3,"       (V)       (M)           (M-V)    (T)      (E)            (M-V+T)                   \n");
      fprintf(fp3,"________________________________________________________________________________________________\n");

   
      if(n_value == 1000) z001=0;   else z001=1;  /* set to = 0 if smallest z false positive = 0.001 */
                                                  /* this is to keep -z 6 = 0.75 false positives     */
      for(k=0; k<RANK_SFZ; k++)
      {
          sd=std_dev(s_value,Rcor[0][k],Rcor[1][k]);
          se=sqrt(sd*sd-sum_vw);
         /* if matches found < false positives */
         if((ngood[0][k]/s_value - alphaCut[k]) < mls_cutoff)
         {
            if(k == z_value) fprintf(fp3,"%3d# ",k+z001);
            else             fprintf(fp3,"%3d  ",k+z001);
            if(alphaCut[k] < 0.15) fprintf(fp3," %6.3f    ",alphaCut[k]);
            else                   fprintf(fp3,"%6.2f     ",alphaCut[k]);
            fprintf(fp3,"%6.2f (%5.2f) ",(ngood[0][k]/s_value),std_dev(s_value,ngood[0][k],ngood[1][k]));
            fprintf(fp3,"%6.2f   ",(R[0][k]/s_value));
            fprintf(fp3,"%6.2f  ",0.0);
            fprintf(fp3,"%6.2f (%5.2f) ",(Ex[0][k]/s_value),std_dev(s_value,Ex[0][k],Ex[1][k]));
            fprintf(fp3,"%6.2f (%5.2f)",0.0,se);
            fprintf(fp3,"%6.2f (%5.2f) ",0.0,0.0);
            fprintf(fp3,"   Warning: Shadows > Matches \n");
         }
         else
         {
            if(k == z_value) fprintf(fp3,"%3d# ",k+z001);
            else             fprintf(fp3,"%3d  ",k+z001);                                                   /* z */
            if(alphaCut[k] < 0.15) fprintf(fp3," %6.3f    ",alphaCut[k]);
            else                   fprintf(fp3,"%6.2f     ",alphaCut[k]);                                   /* V */
            fprintf(fp3,"%6.2f (%5.2f) ",(ngood[0][k]/s_value),std_dev(s_value,ngood[0][k],ngood[1][k]));   /* M */
            fprintf(fp3,"%6.2f   ",(R[0][k]/s_value));                                                      /* M-V */
            fprintf(fp3,"%6.2f  ",(Rcor[0][k]/s_value)-(R[0][k]/s_value));                                  /* T */
            fprintf(fp3,"%6.2f (%5.2f) ",(Ex[0][k]/s_value),std_dev(s_value,Ex[0][k],Ex[1][k]));            /* E */
            fprintf(fp3,"%6.2f (%5.2f)",(Rcor[0][k]/s_value),se);       /* corr recap */
            if(k == kMin) fprintf(fp3,"*");
            else          fprintf(fp3," ");
            fprintf(fp3,"%6.2f (%5.2f) \n",(pctR[0][k]/s_value),std_dev(s_value,pctR[0][k],pctR[1][k]));    /* % recap */
         }

      }
      fflush(fp3);

      /* test if z_value converged with this ci iteration */
      if(z_done_before(kMin))
      {
         if(kMin == (best_z=find_best_z()) )
         { /* converged */
           z_converge_note(0,kMin);
           return(z_value);  /* returning z_value will terminate ci loop in main */
         }
         else
         {
           if(kMin > best_z) new_z=best_z+1;
           else              new_z=best_z-1;

           if(z_done_before(new_z))
           { /* converged */
             best_z=find_best_z();
             z_converge_note(1,best_z);
             return(z_value);  /* returning z_value will terminate ci loop in main */
           }
           else kMin=new_z;
         }
      }
      z_warning(kMin,z_value); /* warning if need to re-run analysis */

   }
   else
   {
      fprintf(fp3,"   There were no matches \n");
   }

   return(kMin);  /* internal z_value starts at zero */
}
