#include        <math.h>       /* pow sqrt                          */
#include        "shaza.h"      /* for shaza structs and defines etc */
#include        "command.h"    /* for shaza command line        etc */

/*************************************************************************************/
double get_se(ia,ib,rBlocks,sum_X,sum_XX,sum_vw)
/* returns standard error of recapture estimate adjusted for whole number sampling */
/* coded by: Michael Macbeth       revision - 25/10/2010     */
int ia;
int ib;
int rBlocks;
double sum_X;
double sum_XX;
double *sum_vw; /* sum of variance attributed to sampling recaptures as whole numbers */
/*************************************************************************************/
{
   double p;   /* fraction of recapture simulated e.g. if 3.4 recaps simulated p=0.4 */
   double sd;  /* standard deviation */
   double vw;  /* vw=additional variance due to sampling recaptures as whole numbers */
   double x;   /* se variance */
   int m;

  /* p=fraction of recapture simulated: =0.0 if no recap simulated for ia*ib block */
  /* can only simulate whole numbers of recaptures so round up or down to get average recapture */
  p=0.0;
  for(m=0; m<rBlocks;m++)
  if((ib == simBlock[m].ia)&&(ia == simBlock[m].ib)) /* this is correct as ia and ib the other way around */
  {                                                  /* important when searching lower diagonal           */
    p=simBlock[m].up/(double)s_value;
  }
  vw=s_value*p*(1-p)/(s_value-1); 
  *sum_vw=*sum_vw+vw;

  if(s_value > 1)
  {
    /*          n       sum(X)sum(X*X) */
    sd=std_dev( s_value,sum_X,sum_XX);

    if(p>SMALL)
    {
       if(  (x=(sd*sd-vw)) > 0.0) return(sqrt(x));
       else                       return(0.0);
    }
    else return(sd);
  }
  else return(0.0);

}
