/* coded by: Michael Macbeth       revision - 28/06/2011     */
#include <stdio.h>
#include <time.h>

#define extern
#include "command.h"
#undef extern
#include "shaza.h"

/* Operating system:  */
#define OSYS        1   /* =1 if Microsoft Windows PC */
                        /* =0 if Linux/Mackintosh PC  */


void        init_genrand();
void        init_by_array();
void        program_name();              /* takes directory prefix out of program name */
void        finish_time();               /* display finish time in minutes and seconds */
int         which_data_format();         /* assigns f_value */
int         command_line();              /* defines input variables from command line */
int         shaza();

/**************************************************************/
int main(argc,argv)
int argc;
char *argv[];
/**************************************************************/
{
  char program[BUFSIZ];
  int model;
  unsigned long clock_start;  /* sizeof 'unsigned long' same as 'int' in MSPC */
  unsigned int iseed;         /* random seed number */
  int ci=0;                   /* convergence iteration */
  int z_value_in;
  int z_value_out;

  clock_start=clock(); /* set start time to estimate elapse time */
                       /* if clock() > MAX_INT it will restart at zero */

  /* all command line arguments defined outside SHAZA code */
  program_name(program,argv[0]);
  /* define extern valiable arguments for shaza() run */
  if((model=command_line(&argc,&argv,program)) >0)
  {
    /* set random seed */
    rand_seed(model,&iseed);

    /* open files for writing */
    open_files(model,iseed);

    /* auto detection of data format in base pair file */
    if(i_option[0]) f_option=which_data_format();

    z_value_in=z_value;
    z_value_out=shaza(program,model,iseed,z_value_in,ci);  /* start SHAZA */

    /* if not printing simulated geneotype file */
    if(!p_option)
    {
      if(c_value > 1)  /* convergence iteration */
      for(ci=1; ci< c_value; ci++)
      {
        if(z_value_out != z_value_in)
        {
          z_value_in=z_value_out;
          z_value_out=shaza(program,model,iseed,z_value_in,ci);   /* re-run SHAZA */
        }
        else  ci=c_value; /* treminate i loop */
      }
    }
    finish_time(clock_start,program);
  }
  else
  {
    fprintf(stdout,"Please read full SHAZA documentation (Options are case sensitive)\n");
  }
  close_files(model);
  return(1);
}


/*************************************************************************************/
void rand_seed(model,iseed)
int model;
unsigned int *iseed;
/* coded by: Michael Macbeth       revision - 5/10/2009     */
/* initiates random seed */
/*************************************************************************************/
{

   unsigned long init[4]={0x123, 0x234, 0x345, 0x456}, length=4;
   /* initialise array */
   init_by_array(init, length);

   /* initialise random number seed */
   if(r_option)   *iseed=(unsigned int)r_value;
   else           *iseed=(unsigned int)time(NULL);

   init_genrand(*iseed);

}


/**************************************************************/
void program_name(program,name)
char program[BUFSIZ];
char name[BUFSIZ];
/* Take path prefix out of program name */
/* Michael Macbeth 18/11/2009 */
/**************************************************************/
{
   int i,len,pos=0;
#if OSYS
   char c1='\\';    /* if MS Windows DOS etc */
#else   
   char c2='/';     /* if SUN OS, UNIX Linux or other UNIX derivative */
#endif

     len=strlen(name);
     for(i=(len-1); i>=0; i--)
        if((name[i] == c1 )||(name[i] == c1 )) { pos=i+1; i=-1; }

     for(i=pos; i<=len; i++)
        program[i-pos]=name[i];
}

/*************************************************************************************/
int which_data_format()
/* coded by: Michael Macbeth       revision - 25/08/2010                  */
/* this program defines f_option as: 0 csv format (from spreadsheet)      */
/*                                   1 GENEPOP format                     */
/*************************************************************************************/
{
   int i=-1;
   int line=0;
   int comma=0;
   int len;
   char aa[STR2];

   /* open file for reading */
   if( (fp_in = fopen(i_value[0], "r" )) == NULL )
   {
      fprintf(stdout,"which_data_format: Cannot open <%s> for reading\n",i_value[0]);
      exit(1);
   }

   while( fgets(aa,STR2,fp_in) != NULL) /* get one line at a time */
   {
      line++;

      /* check one */
      if(line == 4)
      {
        fclose(fp_in);

        len=strlen(aa);

        for(i=0; i< len; i++)
        if( aa[i] == ',') comma++;

        if(comma == 1) return(0);   /* GENEPOP format as only one comma expected in base pair data line */
        else
        if(comma > 1)  return(1);   /* else SHAZA  format */
        else fprintf(stdout,"No comma found in line 4 and cannot detect if your data is in GENEPOP or SHAZA format.\nPlease read manual.\n");

      }
   }
   fprintf(stdout,"Number of lines read in file %s was %d\nThis is too few lines and program is exiting for you to check",i_value[0],line);
   fprintf(fp7,"Number of lines read in file %s was %d\nThis is too few lines and program is exiting for you to check",i_value[0],line);
   exit(1);
}
