#include "shaza.h"
#include "command.h"

/*************************************************************************************/
void read_allele_freq(nloci)
/* coded by: Michael Macbeth       revision - 26/09/2010     */
/*************************************************************************************/
int *nloci;   /* number of loci */
{                      /* this program checks data entry         */
   char aa[STR];
   char a[STR];
   int i;
   int j;
   int locus=0, allele=0;
   int line=0;
   int field=0;
   double tfreq,x;
   int check_once=1;
   int name=1;        /* 1 read allele name OR 0 read allele frequency */
   int short_type=0;  /* Two format types read which are automatically detected */
                      /* long has heading line with base pairs:                Loci_name_1 basepair_1 basepair_2 ... basepair_n */
                      /* long also has frequency line for each Loci_name                                                        */
                      /* short has frequency line only:                        Loci_name_1 freq1       freq2     ... freqn      */


   /* open file for reading */
   if( (fp_in = fopen(i_value[1], "r" )) == NULL )
   {
      fprintf(fp7,"read_allele_freq: Cannot open input file %s\n",i_value[1]);
      fprintf(stdout,"read_allele_freq: Cannot open input file %s\n",i_value[1]);
      exit( -1 );
   }


   while( fgets(aa,STR,fp_in) != NULL) /* get one line at a time */
   {
      line++;
      field=0;


      if(strlen(aa) >= STR )
      {
         fprintf(fp7,"Need to increase size of STR=%d to %d so all characters can be read in line %d of file %s\n",STR,strlen(aa),locus+1,i_value[1]);
         fprintf(stdout,"Need to increase size of STR=%d to %d so all characters can be read in line %d of file %s\n",STR,strlen(aa),locus+1,i_value[1]);
         exit(1);
      }
      else if(strlen(aa) > 1)
      {

         tabtocsv(aa);   /* convert tab separated field to csv */
         csver(aa);      /* prepare cvs format */

         field++;
         rem_blank(aa); /* remove heading blanks */
         sscanf(aa,"%s",loc[locus].name); /* get locus name */
         rem_string(aa); /* remove heading number or string */
         rem_blank(aa);  /* remove heading blanks */

         /* check format type */
         /* Assumes short type if there is a decimal place in second field of first record */
         if(check_once)
         {
           for( j=0; j< ((int)strlen(aa)-1); j++)
           if(aa[j] == '.' )
           {
             short_type=1;
             name=0;
             j=(int)strlen(aa);
           }
           check_once=0;
         }

         if(name)
         {
           name=0;
           allele=0;
           tfreq=0.0;

           if(++locus > NOL )
           {
              fprintf(fp7,"Need to increase dimension for number of loci (NOL=%d) \n",NOL);
              fprintf(stdout,"Need to increase dimension for number of loci (NOL=%d) \n",NOL);
              exit(1);
           }

           while( strlen(aa) > 1 )   /* strlen() counts '\n' */
           {
              sscanf(aa,"%s",a);     /* get next base pair number */
              field++;
     
              loc[locus-1].allele[allele]=check_int(a,line,field,i_value[1]);

              loc[locus-1].n=++allele; 
              if(allele > NOA )
              {
                 fprintf(fp7,"Need to increase size of NOA=%d to read more alleles \n",NOA);
                 fprintf(stdout,"Need to increase size of NOA=%d to read more alleles \n",NOA);
                 exit(1);
              }
     
              rem_string(aa); /* remove heading number */
              rem_blank(aa);  /* remove heading blanks */
           }

         }
         else
         {
           if(short_type) locus++;
           else           name=1; /* read base pairs next time */
           
           allele=0; 
           tfreq=0.0;

           if(locus > NOL )
           {
              fprintf(fp7,"Need to increase dimension for number of loci (NOL=%d) \n",NOL);
              fprintf(stdout,"Need to increase dimension for number of loci (NOL=%d) \n",NOL);
              exit(1);
           }
     
           while( strlen(aa) > 1 )   /* strlen() counts '\n' */
           {
              sscanf(aa,"%s",a);     /* get next allele frequency number */
              field++;
     
              loc[locus-1].freq[allele]=check_double(a,line,field,i_value[1]);
              tfreq=tfreq+loc[locus-1].freq[allele];
              loc[locus-1].n=++allele;

              /* allele number from 1 as zero used to identify missing allele */
              if(short_type) loc[locus-1].allele[allele-1]=allele;

              if(allele > NOA )
              {
                 fprintf(fp7,"Need to increase size of NOA=%d to read more alleles \n",NOA);
                 fprintf(stdout,"Need to increase size of NOA=%d to read more alleles \n",NOA);
                 exit(1);
              }

              rem_string(aa); /* remove heading number */
              rem_blank(aa);  /* remove heading blanks */
           }

           /* calculate cumulative frequencies */
           x=0.0;
           if(allele > 0 )
           for(i=0; i < allele; i++)
           {
             x=x+loc[locus-1].freq[i];
             loc[locus-1].cfreq[i]=x/tfreq;
           }
           if((x < 0.99)||(x > 1.01))
           {
             fprintf(stdout,"\nread_allele_freq: Allele frequecies in line=%d do not add up to one (total frequency is %f)\n",line,x);
             fprintf(stdout,"                  Please check input file <%s>.\n",i_value[1]);
             exit(1);
           }

         }
      }
   }

   fclose(fp_in);
   *nloci=locus;
}


