#include "shaza.h"
#include "command.h"
/*************************************************************************************/
void read_base_pairs(nloci,nrecords,num_block,anim)
/* coded by: Michael Macbeth       revision - 5/10/2009     */
/* this program reads and checks locus and allele data                               */
/*************************************************************************************/
int *nloci;       /* number of loci read */
int *nrecords;    /* number of records read */
int *num_block;   /* number of blocks of data */
int  anim;        /* maximum number of genotype samples that can be read */
{
   char aa[STR];
   char bb[STR];
   char a[SOL];
   char b[SOL];
   char c[STR];
   char missing[2];
   int first_line=0;
   int nanimal=0;
   int locus=0;
   int num;
   int loci_pairs=0;
   int check_loci;
   int n_withinBlock[BLOCK];  /* number within each block */
   int first_block=0;         /* indicator if first block read */
   int block=-1;
   int i,j;
   int maxlocus=0;
   unsigned int bit1, bit2, bit3=0;
   int line=0;
   int field=0;
   int found=0;


   missing[0]=(char)MISSINGc;
   missing[1]='\0';

   /* open file for reading */
   if( (fp_in = fopen(i_value[0], "r" )) == NULL ) fatal(2,i_value[0]," "," "," ",0,0,0);

   while( fgets(aa,STR,fp_in) != NULL) /* get one line at a time */
   {
      line++;
      field=0;

      /* check if string was long enough to detect end of line */
      for(j=0; j<STR; j++) if(aa[j] == '\n')  found=1;
      if(!found) fatal(3,i_value[0]," "," "," ",STR,line,0);

      if(strlen(aa) > 1)   /* ignore blank lines */
      {
         strcpy(bb,aa);
         csver(aa); /* prepare cvs format and fill in MISSING values */

         if(first_line) maxlocus=locus;
         locus=0;

         if(first_line == 0)  /* process heading line */
         {
            first_line++;
            field++;
            rem_blank(aa);
            rem_string(aa); /* remove animal column title (not used) */
            rem_blank(aa);

            while( strlen(aa) > 1 )  /* counts \n */
            {
               if(has_char(aa,' ') >= SOL) fatal(4,aa," "," "," ",SOL-1,line,0);

               /* get locus name */
               sscanf(aa,"%s",a);

               if(locus >= NOL ) fatal(5," "," "," "," ",NOL,0,0);

               strcpy(loc[locus++].name,a);
               rem_string(aa);
               rem_blank(aa);

               /* get next allele name */
               if(strlen(aa) > 1 )
               {
                 sscanf(aa,"%s",b);
                 if(strcmp(a,b))
                 {
                   fprintf(fp7,"          Pair of allele names do not match (%s) (%s) in first line of data. locus=%d\n",a,b, locus);
                   fprintf(fp7,"          Make sure data is in SHAZA format or use '-f' option to define GENEPOP data format \n\n");

                   fprintf(stdout,"          Pair of allele names do not match (%s) (%s) in first line of data. locus=%d\n",a,b, locus);
                   fprintf(stdout,"          Make sure data is in SHAZA format or use '-f' option to define GENEPOP data format \n\n");
                   exit(1);
                 }
                 rem_string(aa);
                 rem_blank(aa);
                 loci_pairs++;
               }
               else fatal(6,a,bb," "," ",line,0,0);

            }
            locus=0;
            if(!loci_pairs) fatal(7," "," "," "," ",0,0,0);

            /* define bits for loci used */
            bit1=~0;
            bit2=0;
            for(i=loci_pairs; i < 32; i++)
            bit2=bit2|(1<<(i));     /* loci bits not used */
            bit3=bit1^bit2;         /* only loci bits used have bit=1 */
                                    /* eg for 32 bit machine with 12 loci bit3 = 00000000000000000000111111111111 */
         }
         else  /* process non-heading line (BLOCK statement line or allele lines) */
         {

            rem_blank(aa);

            /* check string dimension */
            if(has_char(aa,' ') >= SOA) fatal(8,aa," "," "," ",SOA-1,line,0);

            /* get sample name */
            field++;
            sscanf(aa,"%s",rec[nanimal].name); /* scan animal name or number as string */

/* if(!strcmp(",",bb))    assume not a new block if have comma on line */
                                /* this is to avoid sample name having the block name as a prefix */


            if(!strcmp(k_value,rec[nanimal].name)) /* if new block */
            {

              if(m_option)
              {
                if((block > 0)&&(n_withinBlock[block] <  m_value))
                {
                  fprintf(fp7,"Block %s deleted as it had < m_value=%d\n",b_name[block],m_value);
                  fprintf(stdout,"Block %s deleted as it had < m_value=%d\n",b_name[block],m_value);
                  nanimal=nanimal-n_withinBlock[block];
                }
                else block++;
              }
              else block++;
              if(block >= BLOCK ) fatal(9," "," "," "," ",BLOCK,0,0);

              first_block=1;
              n_withinBlock[block]=0;
              nBlock[block]=nanimal;

              rem_string(aa);    /* removes block separator k_value */
              rem_blank(aa);
              field++;
              sscanf(aa,"%s",b_name[block]); /* scan block name or number as string */

              if((block > 0)&&(n_withinBlock[block-1] < 1)) fatal(15,k_value," "," "," ",block,n_withinBlock[block-1],0);   /* Make sure there are >=1 samples per block */
            }
            else /* not new block */
            {

              if(!first_block)  fatal(1,k_value," "," "," ",block,0,1);

              n_withinBlock[block]++;
              rec[nanimal].block=block;
              rec[nanimal].mbit=bit3;    /* i.e. initialise with no loci missing */

              rem_string(aa);
              rem_blank(aa);

              check_loci=0;
              /* read all alleles for every animal */
              /*while( strlen(aa) > 2 )    counts \n */
              while(strlen(aa) > 1)
              {
                 /* get first allele */
                 field++;
                 sscanf(aa,"%s",a);
                 animal_data[nanimal][locus][0]=num=check_int(a,line,field,i_value[0]);
                 rem_string(aa);
                 rem_blank(aa);

                 /* get second allele */
                 if(strlen(aa) > 1 )
                 {
                   field++;
                   sscanf(aa,"%s",b);
                   animal_data[nanimal][locus][1]=check_int(b,line,field,i_value[0]);
                   rem_string(aa);
                   rem_blank(aa);
                   check_loci++;
                 }
                 else  fatal(10,a,bb," "," ",line,0,0);

                 if(((!strcmp(a,missing))||(!strcmp(b,missing)))&&(strcmp(a,b))) 
                 {
                    if(strcmp(a,missing))  sscanf(a,"%s",c);
                    else                   sscanf(b,"%s",c);
                    fatal(12,c,rec[nanimal].name,loc[locus].name,bb,nanimal+block+2,0,0);
                 }

                 /* if missing store missing loci in bitwise array */
                                 /* unsigned int one=1,mis;                           */
                                 /* mis=(one<<locus);                                 */
                                 /* one=1;                                            */
                                 /* rec[nanimal].mbit=rec[nanimal].mbit^(one<<locus); */
                 if(num == 0 )    rec[nanimal].mbit^=(1<<locus);   /* does all above   */
                 if((locus++)  >= NOL  ) fatal(11," "," "," "," ",NOL,0,0);
              }
              if(check_loci != loci_pairs) fatal(13,rec[nanimal].name,loc[locus].name,bb," ",check_loci,loci_pairs,nanimal+block+2);
              if(++nanimal > anim )        fatal(14,bb," "," "," ",ANIM,nanimal,0);
            }
         }
      }
   }


   nBlock[block+1]=nanimal;

   *nloci=maxlocus;
   *nrecords=nanimal;
   *num_block=block+1;    /* return range in block count from 1...n */

   fclose(fp_in);
}


/*************************************************************************************/
void fatal(err,str_1,str_2,str_3,str_4, num_1,num_2,num_3)
/*************************************************************************************/
int err;
char *str_1;
char *str_2;
char *str_3;
char *str_4;
int num_1;
int num_2;
int num_3;
{

  if(err == 1)
  {
     fprintf(stderr,"\n          In '-1 FILE' must define population structure immediately after allele header line with loci names\n");
     if(num_2 < 1)
     fprintf(stderr,"          It appears that population number %d has no samples. Is this correct?\n",num_1);
     if(num_3 == 1)
       fprintf(stderr,"          It appears that population separator <%s> was not read in data. Please check.\n",str_1);
     if(k_option)
       fprintf(stderr,"          The population separator defined in command line using '-k' was <%s>. Is this correct?\n",str_1);
     else
       fprintf(stderr,"          Is the population separator POP used in your data. If not change using -k option.\n");
     fprintf(stderr,"          \n\n [name_of_population] is optional.  Example run file: \n \n");
     fprintf(stderr," This first line is required and can contain a description of data contents\n");
     fprintf(stderr," Locus1, Locus2, Locus3 \n");
     fprintf(stderr," POP [name_of_population_1]\n");
     fprintf(stderr," Sample_1,      200203 301320 300303\n");
     fprintf(stderr," Sample_2,              ...\n");
     fprintf(stderr," .                      ...\n");
     fprintf(stderr," Sample_(n),            ...\n");
     fprintf(stderr," POP [name_of_population_2]\n");
     fprintf(stderr," Sample_(n+1),          ...\n");
     fprintf(stderr," Sample_(n+2),          ...\n\n");
  }
  else
  if(err == 2)
  {
     fprintf(stderr, "read_base_pairs: Cannot open input file <%s>\n",str_1);
     fflush(stderr);
  }
  else
  if(err == 3)
  {
    fprintf(stderr,"Need to increase size of STR=%d and recompile to read to end of line in file %s\n",num_1,str_1);
    fprintf(stderr,"Last line read was %d\n",num_2);
  }
  else
  if(err == 4)
  {
    fprintf(stderr,"If GENEPOP format used check if 'POP' is at beginning of third line.\n");
    fprintf(stderr,"Else if SHAZA format:\n");
    fprintf(stderr,"  Loci names can only be read with %d characters. \n",num_1);
    fprintf(stderr,"  Need to reduce length of locus name OR increase size of SOL and recompile\n");
    fprintf(stderr,"  Last line read was: %d\n <%s>",num_2,str_1);
    fprintf(stderr,"Else check that your data has the correct format (see SHAZA manual).\n");
  }
  else
  if(err == 5)
  {
    fprintf(stderr,"Need to increase size of NOL=%d to read more loci\n",num_1);
  }
  else
  if(err == 6)
  {
    fprintf(stderr,"Read first allele but could not find second allele (%s)\n",str_1);
    fprintf(stderr,"Line number %d\nLast line read was:\n %s \n",num_1,str_2);
  }
  else
  if(err == 7)
  {
    fprintf(stderr,"No loci pairs were read in first line\n"); 
  }
  else
  if(err == 8)
  {
    fprintf(stderr,"Sample names can only be read with %d characters. \n",num_1);
    fprintf(stderr,"Need to increase size of SOA and recompile OR reduce length of name\n");
    fprintf(stderr,"Last line read was: %d\n <%s>",num_2,str_1);

  }
  else
  if(err == 9)
  {
    fprintf(stderr,"Too many populations read in dataset\n");
    fprintf(stderr,"Maximum populations in this compiled program is:  BLOCK=%d\n",num_1);
    fprintf(stderr,"Need to increase BLOCK parameter in shaza.h and recompile\n");
  }
  else
  if(err == 10)
  {
    fprintf(stderr,"Read first allele number (%s) but could not find second allele number \n",str_1);
    fprintf(stderr,"Line number %d\nLast line read was:\n %s \n",num_1,str_2);
  }
  else
  if(err == 11)
  {
    fprintf(stderr,"read_base_pairs: Need to increase size of NOL=%d to read more loci\n",num_1);
    /* TODO count max NOL in data and report before exiting */
  }
  else
  if(err == 12)
  {

    fprintf(stderr,"\n\n___________________________________\nAllele pairs must both be missing\n___________________________________\nNon-missing allele is:");
    fprintf(stderr," <%s>\n",str_1);
    fprintf(stderr,"       Animal name is: %s\n",str_2);
    fprintf(stderr,"       Locus  name is: %s\n",str_3);
    fprintf(stderr,"       Line number is:%4d\n",num_1);
    fprintf(stderr,"       Line read is: %s\n",str_4);
  }
  else
  if(err == 13)
  {
    fprintf(stderr,"\n\n_______________________________________________________________________________________\n");
    fprintf(stderr,"Loci pairs (%d) in current line do not match in number with those in header file (%d)\n",num_1,num_2);
    fprintf(stderr,"_______________________________________________________________________________________\n");
    fprintf(stderr,"       Animal name is: %s\n",str_1);
    fprintf(stderr,"       Locus  name is: %s\n",str_2);
    fprintf(stderr,"       Line number is:%4d\n",num_3);
    fprintf(stderr,"       Line read is: %s\n",str_3);
  }
  else
  if(err == 14)
  {
    fprintf(stderr,"read_base_pairs: Need to increase size of ANIM=%d to read more genotype records\n",num_1);
    fprintf(stderr,"   Last line read was : %s",str_1);
    fprintf(stderr,"   Samples read so far: %d",num_2);
    /* TODO count ANIM in data and report before exiting */
  }
  else
  if(err == 15)
  {
     fprintf(stderr,"\n          Make sure data is in SHAZA format or use '-f' option to define GENEPOP data format \n\n");
  }
  fflush(stderr);
  exit(1);
}


