#include "command.h"
#include "shaza.h"
/*************************************************************************************/
void read_base_pairs_GENEPOP(nloci,nrecords,num_block,anim)
/* coded by: Michael Macbeth       revision - 5/10/2009     */
/* this program reads and checks locus and allele data in GENEPOP data format        */ 
/*    25/08/2010 fixed problem with sample name heading same as block name     */
/*************************************************************************************/
int *nloci;       /* number of loci read */
int *nrecords;    /* number of records read */
int *num_block;   /* number of blocks of data */
int  anim;        /* maximum number of genotype samples that can be read */
{
   char aa[STR];
   char bb[STR];
   char cc[STR];
   char a[SOL];
   char missing[2];
   int first_line=0;
   int nanimal=0;
   int num1, num2;
   int n_withinBlock[BLOCK];  /* number within each block */
   int first_block=0;         /* indicator if first block read */
   int block=-1;
   int i,j,k=1;
   unsigned int bit1, bit2, bit3=0;
   int line=0;
   int field=0;
   int len;         /* length of string */
   int c_pos;       /* position of character in string */
   int found=0;
   int slen;         

   *nloci=0;
   missing[0]=(char)MISSINGc;
   missing[1]='\0';
   slen=strlen(k_value);

   /* open file for reading */
   if( (fp_in = fopen(i_value[0], "r" )) == NULL ) fatal(2,i_value[0]," "," "," ",0,0,0);

   while( fgets(aa,STR,fp_in) != NULL) /* get one line at a time */
   {
      line++;

      /* check if string was long enough to detect end of line */
      for(j=0; j<STR; j++) if(aa[j] == '\n')  found=1;
      if(!found)  fatal(3,i_value[0]," "," "," ",STR,line,0);


      if(strlen(aa) > 1)      /* ignore blank lines */
      {
         rem_blank(aa); /* remove leading blanks */
         strcpy(bb,aa);

         if((bb[slen] == ' ' )||(bb[slen] == ',' )||(bb[slen] == 10 )) /* 10 is line feed */
         bb[slen]='\0';  /* for 'bb' comparison with k_value */

         if(first_line == 0) first_line++;                  /* ignore first line */
         else
         if((first_block == 0)&&(strcmp(bb,k_value) != 0 ))  /* read allele names */
         {
           while((c_pos=has_char(aa,','))) /* loop to read multiple locus names on one line */
           {
             if(*nloci >= NOL )
             {
                fprintf(fp7,"          If data is in GENEPOP format then possibly need to increase size of NOL=%d and recompile source code to read more loci\n",NOL);
                fprintf(fp7,"          Otherwise check that the population separator using the '-k STR' is correctly defined for the '-1 FILE'.\n");

                fprintf(stdout,"          If data is in GENEPOP format then possibly need to increase size of NOL=%d and recompile source code to read more loci\n",NOL);
                fprintf(stdout,"          Otherwise check that the population separator using the '-k STR' is correctly defined for the '-1 FILE'.\n");
                exit(1);
             }
             if(c_pos >= SOL) fatal(4,aa," "," "," ",SOL-1,line,0);
             for(i=0; i<c_pos; i++) loc[*nloci].name[i]=aa[i];
             loc[*nloci].name[i]='\0';
             *nloci=*nloci+1;
             rem_string_char(aa,','); /* removes string up to and including ',' */
             rem_blank(aa);           /* remove leading blanks */
           }
           if((strlen(aa)-1) >= SOL)  /* check string length of last loci name read */
           fatal(4,aa," "," "," ",SOL-1,line,0);

           if(strlen(aa) > 1)   /* process non-blank lines */
           {
              for(i=0; i<(strlen(aa)-1); i++) loc[*nloci].name[i]=aa[i];
              *nloci=*nloci+1;
           }

           rem_string_char(aa,','); /* removes string up to and including ',' */
           rem_blank(aa);           /* remove leading blanks */
         }
         else /* read alleles */
         {
           if(first_block == 0)
           {
             if(!*nloci) 
             { 
               fprintf(fp7,"Need loci names to be read. Did not read any (nloci=%d)\n",*nloci);
               fprintf(stdout,"Need loci names to be read. Did not read any (nloci=%d)\n",*nloci);
               exit(1);
             }
             first_block=1; /* read at least one allele */

             /* define bits for loci used */
             bit1=~0;
             bit2=0;
             for(i=*nloci; i < 32; i++)
             bit2=bit2|(1<<(i));     /* loci bits not used */
             bit3=bit1^bit2;         /* only loci bits used have bit=1 */
                                     /* eg for 32 bit machine with 12 loci bit3 = 00000000000000000000111111111111 */
           }

           if(strcmp(bb,k_value) == 0 )  /* new block */
           {


              /* define last sample name read as block (population) name */
              field++;
              sscanf(rec[nanimal-1].name,"%s",b_name[block]); /* scan block name or number as string */

              if(m_option)
              {
                if((block > 0)&&(n_withinBlock[block] <  m_value))
                {
                  fprintf(fp7,"Block %s deleted as it had < m_value=%d\n",b_name[block],m_value);
                  fprintf(stdout,"Block %s deleted as it had < m_value=%d\n",b_name[block],m_value);
                  nanimal=nanimal-n_withinBlock[block];
                }
                else block++;
              }
              else block++;
              if(block >= BLOCK ) fatal(9," "," "," "," ",BLOCK,0,0);

              first_block=1;
              n_withinBlock[block]=0;
              nBlock[block]=nanimal;
           }
           else  /* not new block */
           {
              n_withinBlock[block]++;
              rec[nanimal].block=block;
              rec[nanimal].mbit=bit3;    /* i.e. initialise with no loci missing */

              /* check must have comma in string */
              if((c_pos=has_char(aa,',')))
              {
                /* read sample name */
                for(i=0; i<c_pos; i++) rec[nanimal].name[i]=aa[i];
                rec[nanimal].name[i]='\0';
                rem_string_char(aa,','); /* removes string up to and including ',' */
                if(strlen(aa) < 1)
                {
                   fprintf(fp7,"No alleles after sample name (%s)  \n",rec[nanimal].name);
                   fprintf(fp7,"Line number %d\nLast line read was:\n %s \n",line,bb);
                   fprintf(stdout,"No alleles after sample name (%s)  \n",rec[nanimal].name);
                   fprintf(stdout,"Line number %d\nLast line read was:\n %s \n",line,bb);  exit(1);
                }

                /* read alleles for all 'nloci' */
                for(j=0; j < *nloci; j++)
                {
                  rem_blank(aa);    /* remove leading blanks */
                  sscanf(aa,"%s",a);
                  field++;
                  len=strlen(a);

                  if(len == 4) /* 4 intergers */
                  {
                    for(i=0; i<2; i++) cc[i]=aa[i]; cc[2]='\0';
                    animal_data[nanimal][j][0]=num1=check_int(cc,line,field,i_value[0]);  /* first allele */

                    for(i=2; i<4; i++) cc[i-2]=aa[i]; cc[2]='\0';
                    animal_data[nanimal][j][1]=num2=check_int(cc,line,field,i_value[0]);  /* second allele */
                  }
                  else if(len == 6) /* 6 intergers */
                  {
                    for(i=0; i<3; i++) cc[i]=aa[i]; cc[3]='\0';
                    animal_data[nanimal][j][0]=num1=check_int(cc,line,field,i_value[0]);  /* first allele */

                    for(i=3; i<6; i++) cc[i-3]=aa[i]; cc[3]='\0';
                    animal_data[nanimal][j][1]=num2=check_int(cc,line,field,i_value[0]);  /* second allele */
                  }
                  else
                  { 
                    fprintf(fp7,"Must have 4 or 6 integer characters per set of alleles. Read %d integers in line %d.\n",len,line);
                    fprintf(stdout,"Must have 4 or 6 integer characters per set of alleles. Read %d integers in line %d.\n",len,line);
                    exit(1);
                  }

                  /* if missing store missing loci in bitwise array */
                                   /* unsigned int one=1,mis;                           */
                                   /* mis=(one<<locus);                                 */
                                   /* one=1;                                            */
                                   /* rec[nanimal].mbit=rec[nanimal].mbit^(one<<locus); */
                  if((num1==0)||(num2==0))       /* treats a single missing allele as a missing locus */
                  rec[nanimal].mbit^=(1<<j);       /* does all above                */

                  rem_string(aa);     /* remove leading string so can read next allele */
                  if((strlen(aa) == 1)&&(j < (*nloci-1)))  /* if no more alleles but expecting more to be read */
                  {
                    fprintf(fp7,"No. of loci names (%d) is different from No. of alleles (%d) in line (%d)\n",*nloci,j+1,line);
                    fprintf(stdout,"No. of loci names (%d) is different from No. of alleles (%d) in line (%d)\n",*nloci,j+1,line);
                    exit(1);
                  }
                } /* end j loop */

              }
              else
              {
                fprintf(fp7,"Must have comma after sample name then allele number. \n");
                fprintf(fp7,"Line number %d\nLast line read was:\n %s \n",line,bb);
                fprintf(stdout,"Must have comma after sample name then allele numbers. \n");
                fprintf(stdout,"Line number %d\nLast line read was:\n %s \n",line,bb);
                exit(1);
              }

              if(++nanimal > anim ) fatal(14,bb," "," "," ",ANIM,nanimal,0);
           }
         }

      }
      else  /* if blank line is first line - mark first line as read */
      if(first_line == 0) first_line++;                  /* ignore first line */
   }
   /* define last sample name read as block (population) name */
   sscanf(rec[nanimal-1].name,"%s",b_name[block]); /* scan block name or number as string */


   /* check that number within each block is greater than 1 */
   for(i=0; i < block; i++)
   {
     if(n_withinBlock[i] < 1)
     {
        if(k)
        {
           k=0;
           fprintf(fp7,"OK OK\n");
           fprintf(fp7,"Pairwise comparisons need more than 1 animal per data block\n");
           fprintf(fp7,"1 animal per data block is allowed but is only meaningfull when compared to other blocks\n");
           fprintf(fp7,"To help you even more here are the names of data blocks that have < 1 sample in them\n\n");
           fprintf(fp7,"     No.   Name of data block\n");
           fprintf(fp7,"     ________________________________________\n");

           fprintf(stdout,"OK OK\n");
           fprintf(stdout,"Pairwise comparisons need more than 1 animal per data block\n");
           fprintf(stdout,"1 animal per data block is allowed but is only meaningfull when compared to other blocks\n");
           fprintf(stdout,"To help you even more here are the names of data blocks that have < 1 sample in them\n\n");
           fprintf(stdout,"     No.   Name of data block\n");
           fprintf(stdout,"     ________________________________________\n");
        }
        fprintf(fp7,"     %1d     %s\n",n_withinBlock[i],b_name[i]);
        fprintf(stdout,"     %1d     %s\n",n_withinBlock[i],b_name[i]);
     }
     if((i == (block-1))&&(!k))
     {
        fprintf(fp7,"\n\nI just want to let you know that I am not going to CRASH.\n");
        fprintf(fp7,"I am simply going to STOP on my own accord to give you time to correct your data input file.\n");
        fprintf(fp7,"Keep up the good work!\n\nAll the best Michael.\n");

        fprintf(stdout,"\n\nI just want to let you know that I am not going to CRASH.\n");
        fprintf(stdout,"I am simply going to STOP on my own accord to give you time to correct your data input file.\n");
        fprintf(stdout,"Keep up the good work!\n\nAll the best Michael.\n");
        exit(1);
     }
   }

   nBlock[block+1]=nanimal;

   *nrecords=nanimal;
   *num_block=block+1;    /* return range in block count from 1...n */

   fclose(fp_in);
}

