#include "shaza.h"
#include "command.h"
/*************************************************************************************/
void read_data(nloci2,anim2,iperm2,num_block2,nstage,model)
/* coded by: Michael Macbeth       revision - 2/12/2013     */
/* simulates data if allele frequencies read */

int *nloci2;         /* number of loci */
int *anim2;          /* number of genotype samples */
int *iperm2;         /* number of missing loci combinations */
int *num_block2;     /* number of temporal/spatial/treatment groups */
int *nstage;
int model;
/*************************************************************************************/
{
   static int nloci;
   static int anim;
   int anim_L;
   int iperm=1;
   int num_block;      /* number of blocks of data */
   unsigned int        rec_mbit_L[ANIM];               /* store loci missing value combinations for bootstrapping */

   int i;

   if((i_option[0])&&(!i_option[1])) /* if model 1 3 5 7 or 9 */
   {                                 /* read allele alleles from individual animal records (initial data set) */
                                     /* this option does simulate recaptured animals in models 3 5 7 9 */

      /* read locus and allele information into animal_data[] */
      /* also stores missing loci combinations */
      fprintf(fp7,"Reading allele base pairs for match analysis from:            FILE: \"%s\" \n",i_value[0]);
      fprintf(stdout,"Reading allele base pairs for match analysis from:            FILE: \"%s\" \n",i_value[0]); fflush(stdout);

      if(!f_option)  /* GENEPOP data */
      {
        fprintf(fp7,"          GENEPOP format assumed for reading alleles. \n");
        fprintf(stdout,"          GENEPOP format assumed for reading alleles. \n");
        read_base_pairs_GENEPOP(&nloci,&anim,&num_block,ANIM);
      }
      else         /* SPREADSHEET format */
      {
        fprintf(fp7,"          SHAZA spreadsheet format assumed for reading alleles \n");
        fprintf(stdout,"          SHAZA spreadsheet format assumed for reading alleles \n");
        read_base_pairs(&nloci,&anim,&num_block,ANIM);
      }
      fprintf(fp7,"          A total of %d genotype samples were read from %d loci.\n\n",anim,nloci);
      fprintf(stdout,"          A total of %d genotype samples were read from %d loci.\n\n",anim,nloci);
      fflush(stdout);

      /* calculate allele frequencies etc */
      store_loci_freq(nloci,anim); /* store in: loc[].freq[] */
      if(model == 1)
      {
        get_mbit2(anim,&iperm);
      }
      else
      if((model == 3)||(model == 7))
      {
        anim=M_value+H_value;
        if(H_value) num_block=2;
        else        num_block=1;

        nBlock[0]=0;
        nBlock[1]=M_value;
        nBlock[2]=anim;

        bitperm(0,0,1,' ');  /* initialise rec[].mbit */
        get_mbit(anim,nloci,&iperm,Y_value); /* randomly define missing loci combinations for each animal in rec[].mbit        */

        get_reference_data(model,nloci,anim,num_block,*nstage);
        *nstage=*nstage+1;
      }
      else
      if((model == 5)||(model == 9))
      {
        anim_L=anim;
        anim=M_value+H_value;
        if(H_value) num_block=2;
        else        num_block=1;

        nBlock[0]=0;
        nBlock[1]=M_value;
        nBlock[2]=anim;

        /* create new missing loci combinations randomly selected from -1 FILE */
        for(i=0; i< anim_L; i++) rec_mbit_L[i]=rec[i].mbit;   /* Store missing loci combinations of data from -1 FILE */
        bitperm(0,0,1,' ');   /* initialise rec[].mbit */
        get_mbit3(anim,&iperm,anim_L,rec_mbit_L);  /* randomly bootstrap missing loci combinations for each animal */

        get_reference_data(model,nloci,anim,num_block,*nstage);
        *nstage=*nstage+1;
      }
   }
   else /* read allele frequencies from locus records for models 2 4 6 8 or 10 */
   {    /* then generate individual animal records    */
        /* plus any R_value animals                   */

      /* read locus and allele information from -2 FILE */
      fprintf(fp7,"Reading allele frequencies from:                              FILE: \"%s\"\n",i_value[1]);  
      fprintf(stdout,"Reading allele frequencies from:                              FILE: \"%s\"\n",i_value[1]);  fflush(stdout);
      read_allele_freq(&nloci); /* store in: loc[].freq[] */
      
      if(model == 2)
      {
        /* read locus and allele information into animal_data[] */
        /* also stores missing loci combinations */
        fprintf(fp7,"          Reading allele base pairs for match analysis from:            FILE: \"%s\" \n",i_value[0]);
        fprintf(stdout,"          Reading allele base pairs for match analysis from:            FILE: \"%s\" \n",i_value[0]); fflush(stdout);
  
        if(!f_option)  /* GENEPOP data */
        {
          fprintf(fp7,"          GENEPOP format assumed for reading base pairs. \n");
          fprintf(stdout,"          GENEPOP format assumed for reading base pairs. \n");
          read_base_pairs_GENEPOP(&nloci,&anim,&num_block,ANIM);
        }
        else         /* SPREADSHEET format */
        {
          fprintf(fp7,"          SHAZA spreadsheet format assumed for reading base pairs \n");
          fprintf(stdout,"          SHAZA spreadsheet format assumed for reading base pairs \n");
          read_base_pairs(&nloci,&anim,&num_block,ANIM);
        }
        fprintf(fp7,"          A total of %d genotype samples were read from %d loci.\n\n",anim,nloci);
        fprintf(stdout,"          A total of %d genotype samples were read from %d loci.\n\n",anim,nloci);
        fflush(stdout);

        get_mbit2(anim,&iperm);
      }
      else /* models 4 6 8 or 10 */
      {
        /* get missing loci combinations from base pair data or by simulation */
        get_loci_combinations(nloci,model,&iperm,&num_block,&anim);

        get_reference_data(model,nloci,anim,num_block,*nstage);
        *nstage=*nstage+1;
      }

   }
   fprintf(fp7,"\n");
   fprintf(stdout,"\n");

  *nloci2=nloci;
  *anim2=anim;
  *iperm2=iperm;
  *num_block2=num_block;
}


