        /***********************************************************************************************************/
        /*                                                                                                         */
        /* PROGRAM:     SHAZA.c                                                                                    */
        /*              Genotype match assignment using shadow zone                                                */
        /*              Recapture numbers corrected for Type I and Type II errors                                  */
        /*                                                                                                         */
        /*                                                                                                         */
        /* Author:             Michael Macbeth                                                                     */
        /*                     Primary Industries and Fisheries, Queensland                                        */
        /*                                                                                                         */
        /* Acknowledgement:    Damien Broderick who's shadow zone concept lead to the developemnt of this software */
        /*                     Primary Industries and Fisheries, Queensland                                        */
        /*                                                                                                         */
        /*                                                                                                         */
        /*                                                                                                         */
        /*                                                                                                         */
        /***********************************************************************************************************/


#include        <sys/stat.h>   /* used to check if file exists      */
#include        <stdio.h>
#include        <stdlib.h>
#include        <math.h>       /* for pow() sqrt()                  */
#include        <time.h>       /* for clock(), random number seed   */
#include        <string.h>     /* for strlen()                      */

#include        "shaza.h"      /* for shaza structs and defines etc */
#include        "command.h"    /* for command line options          */

void get_alphaCut();

/*************************************************************************************/
int shaza(program,model,iseed,z_value_in,ci)
char *program;
int model;           /* analytical model */
unsigned int iseed;  /* random number seed */
int z_value_in;      /* z_value used in this run Number of false positives=alphaCut[z_value] */
int ci;              /* convergence iteration from 1..n */
/* coded by: Michael Macbeth       revision - 28/06/2011     */
/*************************************************************************************/
{
   static int nloci;           /* number of loci in data              */
   static int anim;            /* number of genotype samples read in data */
   static int iperm;           /* combinations of loci */
   static int nBlocks;         /* number of blocks of data (eg from different time cohorts) */
   static int totalPairs;      /* matches including those Type I error matches above z_value */
   static int goodPairs;       /* matches with false positives below that defined by z_value */
   static double Etotal;       /* effective sample size using all blocks pooled using z_value */
   int rBlocks;                /* number of blocks with recaptures */
   int nstage=1;               /* stage of this computer run */
   int compG;                  /* number of composite groups */
   int z_value_out=0;          /* internal z-value starts at zero  */
   int jj=1;
   double total_corr_recaps;   /* total number of corrected recaptures */

   z_value=z_value_in;
   get_alphaCut();    /* define number of Type I error cutoffs used for each z_value */

   fprintf(stdout,"\n\n__________________________________________________________________\n");
       fprintf(stdout,"Model No: %d   Convergence iteration %d with %5.3f false positives\n",model,ci+1,alphaCut[z_value]);
       fprintf(stdout,"__________________________________________________________________\n\n");
       fprintf(stdout,"STAGE %2d: Listing command line options and defaults\n\n",nstage); fflush(stdout);

   fprintf(fp7,"\n\n__________________________________________________________________\n");
       fprintf(fp7,"Model No: %d   Convergence iteration %d with %5.3f false positives\n",model,ci+1,alphaCut[z_value]);
       fprintf(fp7,"__________________________________________________________________\n\n");
       fprintf(fp7,"STAGE %2d: Listing command line options and defaults\n\n",nstage++);

   if(ci == 0) /* Only read data once during convergence iterations */
   {
     list_arguments(model,iseed,stdout);  /* list option defaults with those changed on command line */
     list_arguments(model,iseed,fp7);     /* list options to SHAZA_log.txt file */

     /* check dimension for machine limits */
     check_dim();

     /* reads allele data OR creates allele data from frequency file */
     fprintf(stdout,"STAGE %2d: ",nstage);  fflush(stdout);
     fprintf(fp7   ,"STAGE %2d: ",nstage++);
     read_data(&nloci,&anim,&iperm,&nBlocks,&nstage,model);


     fprintf(stdout,"STAGE %2d: Create locus/allele frequency report.                         FILE: \"%s\"\n",nstage,o_value[0]);  fflush(stdout);
     fprintf(fp7   ,"STAGE %2d: Create locus/allele frequency report.                         FILE: \"%s\"\n",nstage++,o_value[0]);  fflush(stdout);
     write_loci_freq(nloci,anim,model);

     fprintf(stdout,"STAGE %2d: Report missing loci combinations in reference data.           FILE: \"%s\"\n",nstage,o_value[1]);  fflush(stdout);
     fprintf(fp7   ,"STAGE %2d: Report missing loci combinations in reference data.           FILE: \"%s\"\n",nstage++,o_value[1]);  fflush(stdout);  
     /* write missing value combinations to SHAZA_comb.txt */
     print_missing_loci(nloci,iperm,anim,fp2);   fflush(fp2);


     if(!i_option[1]) /*  if frequency file not provided in command line */
     {
        fprintf(stdout,"STAGE %2d: Create locus/allele frequency file for SHAZA input.           FILE: \"%s\"\n",nstage,o_value[7]);  fflush(stdout);
        fprintf(fp7   ,"STAGE %2d: Create locus/allele frequency file for SHAZA input.           FILE: \"%s\"\n",nstage++,o_value[7]);
        print_loci_freq(nloci,anim);
     }
     else if(p_option) exit(1);


     if((model == 1)||(model == 2))
     write_block_names(nBlocks);

     /* find likelihood zones for cutoff using -n runs */
     fprintf(stdout,"\nSTAGE %2d: Finding false positive distribution using %d simulated data sets.\n",nstage,n_value);
     fprintf(stdout,"          Please wait\n"); fflush(stdout);
     fprintf(fp7   ,"\nSTAGE %2d: Finding false positive distribution using %d simulated data sets.\n",nstage++,n_value);
     get_LZones(model,nloci,anim);

     /* find recaptures from data with given z_value */
     /* needs to be below z_loop to get new estimate of &Etotal */
     fprintf(stdout,"\nSTAGE %2d: Append resample estimates from reference data.                FILE: \"%s\"\n\n",nstage,o_value[4]); fflush(stdout);
     fprintf(fp7   ,"STAGE %2d: Append resample estimates from reference data.                FILE: \"%s\"\n\n",nstage++,o_value[4]);
     get_recaps_from_data(model,nloci,anim,nBlocks,&totalPairs,ci);
   }  /* z_loop */

   goodPairs=(int)d_ngood[z_value];
   Etotal=d_Ex[z_value];


   /* produce a cluster pair report of individual matches  */
   if(goodPairs)
   {
     if(jj)  /* only report mesage once */
     {
       fprintf(stdout,"STAGE %2d: Report pairwise matches to:                                   FILE: \"%s\"\n",nstage,o_value[2]); fflush(stdout);
       fprintf(fp7   ,"STAGE %2d: Report pairwise matches to:                                   FILE: \"%s\"\n",nstage++,o_value[2]); fflush(stdout);
       if(j_option)
       {
         jj=0;
         fprintf(stdout,"STAGE %2d: Write composite genotype file to:                             FILE: \"%s\"\n",nstage,o_value[6]); fflush(stdout);
         fprintf(fp7   ,"STAGE %2d: Write composite genotype file to:                             FILE: \"%s\"\n",nstage++,o_value[6]);
       }
     }
     clusterPairs(nloci,anim,goodPairs,&compG,0);
   }



   /* group recaps from data into blocks and report */
   fprintf(stdout,"STAGE %2d: Report resample percentage within sampling cohorts.           FILE: \"%s\"\n\n",nstage,o_value[3]);  fflush(stdout);
   fprintf(fp7,"STAGE %2d: Report resample percentage within sampling cohorts.           FILE: \"%s\"\n\n",nstage++,o_value[3]);
   total_corr_recaps=dataBlockRecaps(nloci,nBlocks,anim,goodPairs,Etotal,&rBlocks,model,ci);

   if(c_value)
   {
      if(goodPairs)
      {
         /* find recaps from simulation to determine s.e. */
         if(nBlocks > 1)
         {
           fprintf(stdout,"STAGE %2d: Finding s.e. of resample estimates within each population using %d simulation runs.\n",nstage,s_value);
           fprintf(stdout,"          Please wait\n");  fflush(stdout);
           fprintf(fp7,"STAGE %2d: Finding s.e. of resample estimates within each population using %d simulation runs.\n",nstage++,s_value);
   
           z_value_out=get_recaps_from_sim(model,nloci,anim,nBlocks,rBlocks,total_corr_recaps,0,ci);
         }
         else
         {
           fprintf(stdout,"STAGE %2d: Finding s.e. of resample estimate using %d simulation runs.\n",nstage,s_value); 
           fprintf(stdout,"          Please wait\n");  fflush(stdout);
           fprintf(fp7,"STAGE %2d: Finding s.e. of resample estimate using %d simulation runs.\n",nstage++,s_value);
           z_value_out=get_recaps_from_sim(model,nloci,anim,nBlocks,rBlocks,total_corr_recaps,compG,ci);
         }
      }
      else
      {
         fprintf(stdout,"STAGE %2d: Will not find s.e. of resample estimates as no recaptures found\n",nstage);
         fprintf(fp7,"STAGE %2d: Will not find s.e. of resample estimates as no recaptures found\n",nstage++);
      }
   }

   if((z_value_in == find_best_z()) || (!c_value))
   {
     
     shaza_pair_report(model,nloci,anim,totalPairs,ci);

     shaza_pop_report(nBlocks,rBlocks);  

     /* produce a cluster pair report of individual matches  */
     if(goodPairs)
     {
       clusterPairs(nloci,anim,goodPairs,&compG,1); /* appends to SHAZA_pair.txt */
       if(j_option)
       {
         fprintf(stdout,"STAGE %2d: Write composite genotype file to:                             FILE: \"%s\"\n",nstage,o_value[6]); fflush(stdout);
         fprintf(fp7,"STAGE %2d: Write composite genotype file to:                             FILE: \"%s\"\n",nstage++,o_value[6]);
       }
     }
   }

   return(z_value_out);  /* internal z-value starts at zero  */

}


