#define VERSION  "SHAZA Version 2.00"

#define VERBOSE 1        /* 0 short output to stdout */
                         /* 1 long output to stdout */

#define LOOP_SFZ  1000    /* defines dimension used by n_value simulations (n_value defined in command.h) */
                         /* Likelihood runs to determine average cutoff for 1,2,3...S shadows */
                         /* used to define a dimension */
                         /* interactively set on command line using [-n NUM] with maximum value LOOP_SFZ */


#define MULT   1         /* multiple lots of recaptures */

#define USE_R_VALUE 0    /* 0 Use estimated recaptures from reference data to estimate s.d. of recapture estimate */
                         /* 1 Use R_value to estimate s.d. of recaoture estimates */

#define NO_TOT 999999.0  /* LLR too small to convert to alpha using LLRs stored in tot[] dimension */

/* likelihood shadow zone settings */

/* papameters for defining table below: need to see code in get_LZones() for more details */
#define RANK_SFZ  50        /* 50 top likelihoods stored for SZ determination */
#define ALPHA_POW 11        /* 0.0625*2^ALPHA_POW etc = shadow classes */
#define MAX_RANK  RANK_SFZ      /*  maximum rank level for storage of match (shows extra matches outside SFZ) */
#define LLR_TOT  (RANK_SFZ*LOOP_SFZ)



/* simulation dimension settings */

 #define ANIM   30000       /* WARNING: Make sure ANIM is equal to M_MAX in command.h */
                            /* WARNING: 92682 maximum size: see note in check_dim() within shaza.c */
                            /* simulation: ANIM size required is [-N NUM]+[-M NUM]                                              */
                            /* real data: ANIM size required must be > than the number of genotype records in data)             */


/* compiled dimension settings */
/*******************************/

#define PERM     5000       /* max combinations with missing data recorded <=2^NOL-1   (ie PERM=127 for 7 loci) */
                            /* todo:check what happens if perms exceed PERM: best to keep running but with top PERM reported*/
#define BLOCK     100       /* Maximum number of blocks in data set                             */
                            /* Blocks are used to discriminate samples in time cohorts          */
#define PAIRS    6000       /* Dugong  Maximum number in pairwise match list across all blocks                          */




/* other settings */
#define DEBUG      1       /* does extra checks eg sqrt of <=0 etc */



/* output settings */
#define ADDITIONAL  1      /* 0  do not print additional matches that missed out                                           */
                           /* 1  do     print additional matches that missed out                                           */


/* other dimension settings */
#define NOA        80      /* max number of alleles  (38 for genetag data)                                                 */
#define NOL        32      /* max number of loci 32 must not exceed 32 for 32 bit interger                                 */
#define SOL        21      /* max number of characters in loci name in -i1 FILE                                            */
#define SOA        21      /* max number of characters in animal name in -i1 FILE                                          */
#define STR       400      /* max number of characters read per line in -i1 FILE                                           */
#define BLOCKSIZ   80      /* string size (name) used to identify POP in -i1 FILE                                          */
#define PLOID       2      /* 2 for bipliod (must always be 2)                                                             */





/* other internal settings */
#define BLOCKFIT   1000    /* used to store two small ints into one                                                        */
#define MISSINGc   '0'     /* missing alleles defined here must be a character                                             */
#define MISSINGi    0      /* missing alleles defined here must be a single digit                                          */
#define TOL       0.00001  /* tolerance for sum of gene frequencies (for checking only)                                    */
#define SMALL     1.0e-200 /* used to avoid division by negative number in square root or log                              */




/* can include other info eg error rate on individual loci    */
struct locus_id
{
   char     name[SOL];          /* name of locus                                               */
   int      allele[NOA];        /* name of allele in base pair units stored in numeric order   */
                                /* or number of allele if read allele frequency data           */
   int      count[NOA];         /* number of alleles read                                      */
   int      n;                  /* number of alleles at this locus                             */
   int      miss;               /* number of alleles missing at this locus                     */
   double   freq[NOA];          /* frequency at each allele                                    */
   double   cfreq[NOA];         /* cumulative frequency of alleles                             */
};
struct locus_id loc[NOL];       /* alleles[loci number]                                        */


struct anim_rec
{                                                                      /* : defined in         */
  char          name[SOA];      /* name of animal                         : read_base_pairs()  */
  unsigned int  mbit;           /* missing loci storage in bitwise array  : read_base_pairs()  */
  int           block;          /* data block in time/space/treatment cohort                   */
};
struct anim_rec rec[ANIM];



struct perm_rec  /* holds missing allele combination data and statistics */
{                /* only for current block                               */
  unsigned int bit;             /* 32 bits to identify combinations: missing 0, non-missing 1  */
  unsigned int n;               /* count frequency of combination                              */
  char         type;            /*  blank, 'a', 's' or 'e'                                     */
};
struct perm_rec perm[PERM];


struct matches   /* statistics of pairwise matches */
{
  int          pair[2];         /* record number of each pair (from 0...)                      */
  int          block[2];        /*   the block of each pair (from 1..)                         */
  double       LLR;             /* log likelihood ratio of match                               */
  unsigned int bit;             /* missing loci combination of match in bit format             */
};
struct matches mat_pairs[2][PAIRS];  /* first 2 dimension holds [0] for empirical data and [1] for simulation */



struct blockRecap   /* recaptures in blocks */
{
  int          ia;          /* block A                               */
  int          ib;          /* block B                               */
  double       corR;        /* corected recaptures within block      */
  int          up;          /* simulations with corR rounded-up      */
};
struct blockRecap  simBlock[BLOCK];

/* function decalrations TEST */
double              likelihood_ind2();     /* prepare for like_ind() call using animal_data[][][] */
double              likelihood_inds2();    /* prepare for like_ind() call using animal_sim[][][] */
void                likeme2();             /* likelihood of individual match */

/* function decalrations */
double              get_se();                    /* returns s.e. of R_hat corrected for simulation with whole numbers */
void                shaza_pop_report();          /* produces SHAZA_pop.txt file */
int                 basepair_format();           /* returns 0 for GENEPOP format else returns 1 for SHAZA spreadsheet format */
void                z_converge_note2();          /* convergency summary note */
void                z_converge_note();           /* calls z_converge_note2() for different output files */
int                 z_done_before();             /* returns 1 if z_value attempted in ci iteration */
int                 find_best_z();               /* returns z_value with smallest corrected recapture estimate */
void                loci_missmatch();            /* counts missmatching loci */
double              LLR2alpha();                 /* returns estimated number of shadows from distribution of likelihood ranks */
double              LLR_FS();                    /* returns LLR of FS */
double              alpha2LLR();                 /* converts alpha shadows to LLR */
void                z_warning();                 /* re-run with different [-z NUM] warning */
void                get_reference_data();        /* fills animal_data[][][] array */
void                get_loci_combinations();     /* gets missing lici combinations from base pairs or by simulation */
void                list_arguments();            /* list arguments and defaults */
void                print_loci_freq();           /* prints loci frequency in same format read by SHAZA */
void                read_base_pairs_GENEPOP();   /* GENEPOP data input format */
void                print_genotype_data();       /* write genotype data from allele frequencies           */
void                print_genotype_data_GENEPOP();   /* GENEPOP data output format */
void                rem_string_char();           /* removes the first blank deliminated string */
int                 has_char();                   /* returns position of character in string */
void                fatal();                     /* list fatal error messages from reading base pair data */

double              enabled_proportion2();
void                get_blist_rlist();            /* collates blocks with recaptures */
void                get_corrected_match();        /* calculates corrected match using effective size */

int                 ***create3Dint();             /* creates 3 dimensional array using Dynamic Memmory Allocation */
void                free3Dint();                  /* free memory from 3 dimensional array */
void                write_loci_freq();            /* generates loci frequency report */
double              LLR_parent_offspring_E();     /* returns LLR for parent-offspring match for effective size estimation */
double              LLR_parent_offspring_sim();   /* returns LLR for parent-offspring match in simulated data  */
double              effectiveBlockSize(); /* returns effective block size */
int                 compare_alleles();    /* compare allele_base_pair for qsort()                      */
void                skew();               /* finds skewness and std. dev. from X X^2 X^3 data */
double              dataBlockRecaps();    /* write block report of pair-wise matches */
void                simBlockRecaps();     /* simulates recaptures in > 1 blocks and reports in Table 2 SHAZA_pop.txt */
void                simBlockRecaps2();    /* Table 2 report from one block in SHAZA_pop.txt */
void                clusterPairs();       /* finds cluster of matches */
int                 locusBinarySearch();  /* returns position of allele */
int                 inblock();
void                allocate_trio();      /* allocate parent-offspring trio in simulated data */
void                allocate_match();           /* allocates either parent-offspring or individual match in simulated data */
double              Top();                /* transition probability of 1st degree relative (parent-offspring) */
void                likeme();             /* returns various likelihood ratios */
double              enabled_proportion(); /* returns proportion enabled from two samples */
double              std_dev();           /* calculates standard deviation from n sum(x) and sum(x*x)    */ 
void                get_recaps_from_data();  /* finds genotype match's in data */
int                 get_recaps_from_sim();   /* finds genotype match's by simulating data to determine s.e. (returns best z value) */
double              gp_ind();
void                shaza_pair_report(); /* within block pair report */
void                print_sim();         /* prints simulated genotypes */
void                get_LZones();        /* gets likelihood shadow zones */
int                 shaza();
int                 binarySearch();       /* returns position in sorted array */
void                find_like_recap();    /* find recaptures in simulated data using likelihood estimates */
void                find_like_recap_data(); /* find recaptures in reference data using likelihood estimates */
int                 toplikelihood_inds();
double              likelihood_ind();     /* prepare for like_ind() call using animal_data[][][] */
double              likelihood_inds();    /* prepare for like_ind() call using animal_sim[][][] */
double              like_ind();           /* likelihood of individual match */
void                tabtocsv();           /* converts tabs separated fields to csv format */
double              drand();              /* returns double between 0 and 1                               */
int                 randchoice();         /* 50% of time returns either 0 or 1                            */
void                composite();          /* forms composite genotypes                                    */
void                do_rec_search();      /* recursive search of cluster groups of pairwise matches       */
int                 in_one_list();        /* returns 1 is sample already put in onelist                   */
int                 used_one_list();      /* returns 1 if recapture sample already reported               */
void                pair_groups();        /* lists groups of pairwise matches                             */
void                store_match_pairs();  /* stores match pairs found                                     */
void                get_mbit2();          /* stores missing loci combinations */
void                get_mbit3();          /* BOOTSTRAPs non-missing loci from reference data set across all data */
void                get_mbit4();          /* BOOTSTRAPs non-missing loci from reference data set within temporal/spatial group */
void                allelic_dropout2();   /* counts potential allelic dropouts, alleles & no. different*/
void                init_genrand();       /* initialise seed for random number generator               */
void                write_block_names();  /* writes block names provided in data with block number     */
void                read_data();          /* reads base pairs or loci frequency                        */
int                 bitperm();            /* adds unique loci combinations to rec[].mbit               */
void                read_allele_freq();   /* reads allele frequencies from input file                  */
void                read_base_pairs();    /* reads allele base pairs  from input file                  */
void                print_missing_loci();  /* write list of all loci combinations                       */
int                 locus_missing_bit();  /* determines if locus is missing from bit fields            */
double              check_double();       /* return double from string with digits 0 to 9,     or "."  */
int                 check_int();          /* return int    from string with digits 0 to 9              */
double              binsrch();            /* returns frequency of allele at given locus                */
int                 compare_double();     /* for sorting double array                                  */
int                 compare_LLR();        /* compare match struct     for qsort(): ->like              */
void                store_loci_freq();    /* stores loci frequencies in array                          */
void                sim_genotypes();      /* simulate all genotypes from allele frequencies            */
void                sim_genotypes_SIB();      /* simulate FS or HS genotypes             */
int                 sim_individual_match_genotypes();     /* simulate all genotypes from allele frequencies and from matches in data */
void                add_loci_errors();    /* adds SIM_ERROR loci errors per multilocus genotype in recaps   */
void                add_loci_errors_n();  /* adds SIM_ERROR loci errors per multilocus genotype in record n */
void                write_composites();   /* write animal genotypes to file                            */
double              ln_genoprob_bit();    /* returns ln(GP) for given loci combination                 */
void                avge();               /* calculates statistics an vector of double numbers         */
void                check_dim();          /* preliminary check on defined dimensions                   */
void                csver();              /* convert *.csv file into blank spaced file                 */
void                rem_blank();          /* remove blanks from string                                 */
void                rem_string();         /* Remove heading string up to first space character         */
unsigned int        compare();            /* returns match score on pairwise match of biploid alleles  */
void                show();               /* show 01 bits in integer                                   */
void                bitout();             /* called by show to list bits                               */
void                bitout2();            /* list missing locus combinations with '.' and 'M' format   */
void                init_by_array();      /* for WIN random number generator                           */
void                get_mbit();           /* define missing allele combinations in simulated data      */
double              genrand_real2();      /* for WIN random number generator                           */
void                SFZone();             /* find theoretical Shadow Free Zone                         */

int                 animal_sim[ANIM][NOL][PLOID];   /* animal genotype array from simulated data set         */
int                 animal_data[ANIM][NOL][PLOID];  /* animal genotype array from read data                  */
int                 nBlock[BLOCK+1];                /* record number at start of block                       */
int                 one_list[PAIRS*2][2];           /* single vector of matching samples with used indicator */
double              toplikes[RANK_SFZ];             /* top ranked likelihoods                                */
double              toplike[LOOP_SFZ][RANK_SFZ];
double              tot[LLR_TOT];

double cut_av[RANK_SFZ];          /* likelihood shadow cutoff (based on average n th ranked shadow) */
double cut_sd[RANK_SFZ];          /* std. dev. of cut_av[] */

/* simulation data */
double nshad[2][RANK_SFZ];
double ngood[2][RANK_SFZ];        /* total number of matches (including shadows found) */
double R[2][RANK_SFZ];            /* recaptures */
double Rcor[3][RANK_SFZ];         /* corrected recaptures */
double Rse[3][RANK_SFZ];             /* se of corrected recaptures */
double pctR[2][RANK_SFZ];         /* % corrected recaptures */
double PIRs[2][RANK_SFZ];         /* Probability of individual recapture (which is independent of sample size in block) */
double Ex[2][RANK_SFZ];           /* effective sample size */

/* reference data */
double d_ngood[RANK_SFZ];        /* total number of matches (including shadows found) */
double d_R[RANK_SFZ];            /* recaptures */
double d_Rcor[RANK_SFZ];         /* corrected recaptures */
double d_Rse[RANK_SFZ];             /* se of corrected recaptures */
double d_pctR[RANK_SFZ];         /* % corrected recaptures */
double d_PIRs[RANK_SFZ];         /* Probability of individual recapture (which is independent of sample size in block) */
double d_Ex[RANK_SFZ];           /* effective sample size */
int    d_pairs[RANK_SFZ];   
int    d_nrec;

double LLRcut[RANK_SFZ];          /* LLR values at k_th cutoff   */
double alphaCut[RANK_SFZ];        /* alpha values at k_th cutoff */

char b_name[BLOCK][BLOCKSIZ];  /* block name */

int    count[BLOCK][BLOCK];        /* recap number */
double sum_N12[BLOCK][BLOCK];      /* effective block size */
double sum_E12[BLOCK][BLOCK];      /* effective size (E) */
double sum_A[BLOCK][BLOCK];        /* sum shadows (S) */
double sum_rlist[BLOCK][BLOCK];    /* sum matches (M) */
double sd_corR12[BLOCK][BLOCK][2]; /* sum R_hat X XX */

