#include <time.h>
#include "shaza.h"
#include "command.h"
/*************************************************************************************/
/* recapture report                                                                  */
/* within block report */
void shaza_pair_report(model,nloci,anim,totalPairs,ci)
/* coded by: Michael Macbeth       revision - 28/06/2011     */
/*************************************************************************************/
int model;
int nloci;       /* number of loci */
int anim;
int totalPairs;  /* number of total pair-wise matches recorded (including those missed out on alpha threshold) */
int ci;          /* convergence iteration */
{
   int k,n;
   int i_extra=1;
   int no_extra=0;
   int i;
   double shadows;
   char ch=' ';
   int n1, n2;        /* pair samples 1 and 2 */
   int dropout,count,different;
   char itchy;        /* ? or : */
                      /* '?' means that there was >0 alleles different between two pairwise matches */
   time_t time_now;

   time(&time_now); /* get time in seconds */

   fclose(fp5);
   if( (fp5 = fopen(o_value[2], "w" )) == NULL )    
   { fprintf( stderr, "open_files(): Cannot open <S_pair> for writing\n"); exit( -1 ); }

   fprintf(fp5,"%s\n",asctime(localtime(&time_now)));
   fprintf(fp5,"Executable file: %s\n",VERSION);
   if(i_option[0]) fprintf(fp5,"Input data file: %s\n",i_value[0]);
   if(i_option[1]) fprintf(fp5,"Input data file: %s\n",i_value[1]);
                   fprintf(fp5,"Output     file: %s\n\n",o_value[2]);

   /* report simulated recaps */
   if(R_value)
   {
     fprintf(fp5,"\n\n          Simulated relationships:\n");
     fprintf(fp5,"          ___________________________________________\n");
     fprintf(fp5,"          List of pairwise matches\n");
     fprintf(fp5,"          Sample_1               Sample_2\n");
     fprintf(fp5,"          ___________________________________________\n");

     for(k=0; k < MULT;  k++)           /* multiple recaptures    */
     for(n=(anim-R_value); n < anim; n++) /* for n animals          */
     {
      fprintf(fp5,"          %-20s   %-20s\n",rec[n-k*R_value].name,rec[n-anim+R_value].name);
     }
     fprintf(fp5,"          ___________________________________________\n");
   }



   if(totalPairs > 0)
   {
     fprintf(fp5,"\n\n\n\n\n\n\n");
     if(c_value > 1)
     {
       fprintf(fp5,"___________________________________________________\n");
       fprintf(fp5,"Convergence iteration %d with %5.3f false positives\n",ci+1,alphaCut[z_value]);
       fprintf(fp5,"___________________________________________________\n\n");
     }
     fprintf(fp5,"________________________________________________________________________________________________\n");
     fprintf(fp5,"List of pairwise match(s) having cumulative number of false positives <=%-8.3g\n\n",alphaCut[z_value] );

     fprintf(fp5,"________________________________________________________________________________________________\n");
     fprintf(fp5,"Rank      Sample_pairs     Populations      Alleles                     Match log    Cumulative\n");
     fprintf(fp5,"                                            Total Mismatch Dropout      likelihood   number    \n");
     fprintf(fp5,"                                                                        ratio (LLR)  of false  \n");
     fprintf(fp5,"                                                                                     positives \n");
     fprintf(fp5,"________________________________________________________________________________________________\n");

     /* print each pair */
     for(i=0; i< totalPairs; i++)  /* for all pairwise matches within block */
     {
        n1=mat_pairs[0][i].pair[0];
        n2=mat_pairs[0][i].pair[1];

        allelic_dropout2(nloci,n1,n2,&dropout,&count,&different);
        if(different)       itchy='?';
        else                itchy=':';

        shadows=LLR2alpha(mat_pairs[0][i].LLR);
        if( shadows > alphaCut[z_value] )
        {
          if(!w_option)
          {
            no_extra=1;
            if(i_extra)
            {
              fprintf(fp5,  "___________________________________________________\n");
              fprintf(fp5,"\nAdditional pairs that missed out on match criteria:\n");
              fprintf(fp5,  "___________________________________________________\n");
              i_extra=0;
            }
          }
          else no_extra=0;
        } else no_extra=1;

        if(no_extra)
        {

          if(atoi(rec[n1].name) != (atoi(rec[n2].name)+anim-R_value) )
          {
            if((model == 1)||(model == 2))
                 ch=' ';
            else ch='e';   /* match error - determined from simulation data only */
          }
          else   ch=' ';

          if(shadows < (NO_TOT-1) )
          {
            if(((i_option[0])&&(!i_option[1]))||(model == 2) )
            fprintf(fp5,"%3d %10s %c %-10s%c %4d %4d        %3d    %3d   %3d       %8.2f    %10.3f \n",
               i+1,
               rec[n1].name,
               itchy,
               rec[n2].name,
               ch,
               rec[n1].block+1,
               rec[n2].block+1,
               count,
               different,
               dropout,
               mat_pairs[0][i].LLR,
               shadows
               );
            else
            fprintf(fp5,"%3d %10d %c %-10d%c %4d %4d        %3d    %3d   %3d       %8.2f    %10.3f \n",
               i+1,
               n1+1,
               itchy,
               n2+1,
               ch,
               rec[n1].block+1,
               rec[n2].block+1,
               count,
               different,
               dropout,
               mat_pairs[0][i].LLR,
               shadows
               );
          }
          else i=totalPairs;   /* exit loop: other matched have too large LLR to convert to alpha values */
        }
     }
   }
   if(ch == 'e')
   fprintf(fp5,"\n'e' in table above denotes false positive match\n");
   fflush(fp5);
}



