#include        <math.h>       /* pow sqrt                          */
#include        "shaza.h"      /* for shaza structs and defines etc */
#include        "command.h"      /* for shaza command line        etc */

/*************************************************************************************/
void simBlockRecaps(model,nloci,anim,loop,nrec,nBlocks,rBlocks,Etotal)  /* from simulation data with multiple blocks */
/* coded by: Michael Macbeth       revision - 28/06/2011     */
int model;
int nloci;           /* number of loci */
int loop;            /* iteration number of simulation run */
int nrec;            /* number of matches found */
int nBlocks;         /* number of blocks */
int rBlocks;         /* number of blocks in reference data with a recap */
double Etotal;       /* effective block size from total data */
/*************************************************************************************/
{
   int i,j,k;
   int pairs;                   /* pairs with expected false positives < z_value */
   int blocksWithRecaps=0;      /* blocks found with recaptures */
   int blist[BLOCK];            /* block combination of match */
   int rlist[BLOCK];            /* number of recaps in block combination */
   int ia,ib,iaa,ibb;           /* ia ib -block count from 0; iaa ibb -block count from 1 */
   double K;                    /* number of enabled comparisons */
   double kij;                  /* number of anabled comparisons in i j th block */
   double pctR1,pctR2,pctR12;   /* percentage corrected recapture */
   double corR1,corR2,corR12;   /* corrected recapture */
   double PIR1,PIR2,PIR12;      /* Probability of individual recapture */
   double E1=0.0,E2=0.0,E12;    /* Effective sample size */
   int    N1=0,N2=0;            /* block size */
   double N12;                  /* block size */
   double alpha;                /* estimated number of false positives in block */
   double totalNotEnabled;

   static int no_recap_loops;


   totalNotEnabled=anim*(anim-1)/2.0-Etotal*(Etotal-1)/2.0;

   if(loop == 0) no_recap_loops=0;
   if(nrec > 1)
   {
      qsort(mat_pairs[1],nrec,sizeof(struct matches),compare_LLR);
      pairs=0;
      for(i=0; i<nrec; i++)
      if( mat_pairs[1][i].LLR > LLRcut[z_value] ) pairs=pairs+1;  /* count of match pairs within z_value match criteria (other matches rejected) */
   }
   else pairs=0;


   if(pairs == 0)
   {
       no_recap_loops++;
   }
   else
   {
      /* initialise */
      for(i=0; i<BLOCK; i++)   rlist[i]=0;

      if(loop==0)
      for(i=0; i<BLOCK; i++)
      {
         for(j=0; j<BLOCK; j++)
         {
            sum_N12[i][j]=0.0;
            sum_E12[i][j]=0.0;
            sum_A[i][j]=0.0;
            sum_rlist[i][j]=0.0;
            count[i][j]=0;
            for(k=0; k<2; k++)
            {
               sd_corR12[i][j][k]=0.0;
            }
         }
      }

      /* for all accepted matches found in simulated data collate blocks with recaps */
      get_blist_rlist(pairs,blist,rlist,&blocksWithRecaps,1);

      /* calculate within block recap estimates for this TRIAL */
      for(j=0; j<blocksWithRecaps; j++)
      {

         ibb=blist[j]/BLOCKFIT;
         iaa=blist[j]-ibb*BLOCKFIT;
         ia=iaa-1;
         ib=ibb-1;

         if(iaa == ibb)    /* diagonal block */
         {

              N12=(nBlock[iaa]-nBlock[ia]);

              E12=effectiveBlockSize(model,nloci,anim,ia,ib,1,&kij);

              /* stochastic simulation may lead to some variables to cause log problems */
              /* slight adjustments below help make this program more robust */
              if(E12 > N12) E12=N12;
              if(E12 < rlist[j]) E12=rlist[j];

              /* Partition alpha (sum false positives) determined from all data into an estimate of alpha within current block */
              /* This assumes false positives equally likely per not-enabled comparison in different spatial/temporal groups */
              if(totalNotEnabled > 1)
                alpha=alphaCut[z_value]*(N12*(N12-1)/2-kij)/(totalNotEnabled);
              else
              /* If all pairwise comparisons enabled then partition alpha on number of pairwise comparisons in each block */
                alpha=alphaCut[z_value]*(N12*(N12-1)/2)/(anim*(anim-1)/2);


              get_corrected_match(j,iaa,ibb,rlist,alpha,E1,E2,E12,N1,N2,N12,&corR1,&corR2,&corR12,&pctR1,&pctR2,&pctR12,&PIR1,&PIR2,&PIR12);

              /* store records for report */
              if(rlist[j] > 0) count[ia][ib]++;
              sum_N12[ia][ib]=sum_N12[ia][ib]+N12;
              sum_E12[ia][ib]=sum_E12[ia][ib]+E12;
              sum_A[ia][ib]=sum_A[ia][ib]+alpha;
              sum_rlist[ia][ib]=sum_rlist[ia][ib]+rlist[j];   /*  = matches = false positives + recaps */

              sd_corR12[ia][ib][0]=sd_corR12[ia][ib][0]+corR12;               /* sum X  */
              sd_corR12[ia][ib][1]=sd_corR12[ia][ib][1]+corR12*corR12;        /* sum XX  */

         }
         else  /* off-diagonal block */
         {
              N1=nBlock[iaa]-nBlock[ia];
              N2=nBlock[ibb]-nBlock[ib];
              N12=sqrt(N1*N2);    /* effective block size */

              E12=effectiveBlockSize(model,nloci,anim,ia,ib,1,&kij);
              K=E12*E12;
              E1=sqrt(K*N1/N2);
              E2=sqrt(K*N2/N1);

              /* Partition alpha (sum false positives) determined from all data into an estimate of alpha within current block */
              /* This assumes false positives equally likely per not-enabled comparison in different spatial/temporal groups */
              if(totalNotEnabled > 1)
                alpha=alphaCut[z_value]*(N1*N2-kij)/(totalNotEnabled);
              else
              /* If all pairwise comparisons enabled then partition alpha on number of pairwise comparisons in each block */
                alpha=alphaCut[z_value]*(N1*N2)/(anim*(anim-1)/2);

              get_corrected_match(j,iaa,ibb,rlist,alpha,E1,E2,E12,N1,N2,N12,&corR1,&corR2,&corR12,&pctR1,&pctR2,&pctR12,&PIR1,&PIR2,&PIR12);

               /* store records for report */
              /* TODO for memory efficiency do not store in [ia][ib] array - store only blocks with match */
              if(rlist[j] > 0) count[ia][ib]++;
              sum_N12[ia][ib]=sum_N12[ia][ib]+N12;
              sum_E12[ia][ib]=sum_E12[ia][ib]+E12;
              sum_A[ia][ib]=sum_A[ia][ib]+alpha;
              sum_rlist[ia][ib]=sum_rlist[ia][ib]+rlist[j];   /* this includes false positives */

              sd_corR12[ia][ib][0]=sd_corR12[ia][ib][0]+corR12;
              sd_corR12[ia][ib][1]=sd_corR12[ia][ib][1]+corR12*corR12;
         }
      }
   }
}
