#include "shaza.h"
#include "command.h"
/*************************************************************************************/
void sim_genotypes(model,nloci,anim,recap,isim)
/* coded by: Michael Macbeth       revision - 5/10/2009     */
/* simulate genotypes from allele frequencies and adds recap. animals                */
/* NOTE: this defines all loci even those set as missing in rec[].mbit               */
/*************************************************************************************/
int model;
int nloci;      /* number of loci */
int anim;       /* number of genotype samples */
int recap;      /* number of offspring or number of recaptures simulated                   */
int isim;       /* 0 store in: animal_data[] using allele frequencies read in from data OR */
                /* 1           animal_sim[]  to simulate recaptures found                  */
  
{
   int n,i,j,k;
   double xx;
   double terror=E_value/10000.0;     /* terrifying error proportion */

   /* initialise */
   for(i=0; i < NOL; i++)
   for(k=0; k < NOA; k++)
   loc[i].count[k]=0;

   /* Only need to number names when generating an 'original' dataset for analysis */
   /* This is required when frequencies instead of alleles are read in the data */
   if(!isim)
   {
      /* name simulated animal records with number */
      for(n=0; n < anim; n++) /* for n animals          */
      {
         if((model == 1)||(model == 2)) { /* analysis using allele data */ }
         else  sprintf(rec[n].name,"%d",n+1);  /* number animals */

         if(n < M_value) rec[n].block=0; /* simulated block number (only 1 or 2 blocks permitted in 'original' dataset when simulating genotypes) */
         else            rec[n].block=1;
      }
   }

   /* STAGE 1 */
   /* add: 1..(anim-recap) samples */
   for(i=0; i < nloci; i++)               /* for all loci           */
   for(n=0; n < (anim-recap*MULT); n++)   /* for n animals          */
   for(j=0; j < 2;     j++)               /* for haplotypes A & B   */
   {
      xx=drand();
      
      for(k=0; k < loc[i].n; k++) /* for all alleles within locus */
      if(xx <= loc[i].cfreq[k] )
      {
         /* animal_sim[n][i][j]=(k+1);                 number alleles from 1..n */
         if(isim) animal_sim[n][i][j]=loc[i].allele[k];   /* use actual allele number */
         else animal_data[n][i][j]=loc[i].allele[k];
         loc[i].count[k]++;   /* count frequency of alleles used */
         k=loc[i].n;  /* exit loop for faster execution */
      }
   }


   /* STAGE 2 */
   /* add: (anim-recap)..anim duplicate genotype samples */
   if(recap)
   {

     fprintf(fp7,"\n\n          Simulated relationships:\n");
     fprintf(fp7,"          ___________________________________________\n");
     fprintf(fp7,"          List of pairwise matches\n");
     fprintf(fp7,"          Sample_1               Sample_2\n");
     fprintf(fp7,"          ___________________________________________\n");

     fprintf(stdout,"\n          Simulated relationships:\n");
     fprintf(stdout,"          ___________________________________________\n");
     fprintf(stdout,"          List of pairwise matches\n");
     fprintf(stdout,"          Sample_1               Sample_2\n");
     fprintf(stdout,"          ___________________________________________\n");
     for(k=0; k < MULT;  k++)           /* multiple recaptures    */
     for(n=(anim-recap); n < anim; n++) /* for n animals          */
     {
       for(i=0; i < nloci; i++)           /* for all loci           */
       for(j=0; j < 2;     j++)           /* for haplotypes A & B   */
       {
           if(isim) animal_sim[n-k*recap][i][j]=animal_sim[n-anim+recap][i][j];
           else     animal_data[n-k*recap][i][j]=animal_data[n-anim+recap][i][j];
       }
      fprintf(stdout,"          %-20s   %-20s\n",rec[n-k*recap].name,rec[n-anim+recap].name);
      fprintf(fp7,"          %-20s   %-20s\n",rec[n-k*recap].name,rec[n-anim+recap].name);
     }
     fprintf(stdout,"          ___________________________________________\n");
     fprintf(fp7,"          ___________________________________________\n");
   }

   /* STAGE 3 */
   /* add errors to re-samples */
   if(recap)
   add_loci_errors(nloci,1,isim,terror,anim,recap*MULT); /* 1 use:    allele number as loci identification */
}

