#include "shaza.h"
#include "command.h"
/*************************************************************************************/
int sim_individual_match_genotypes(model,nloci,anim,rBlocks,compG,total_corr_recaps)
/* coded by: Michael Macbeth       revision - 10/10/2009     */
/* simulate genotypes from allele frequencies and adds recap. animals with simulated errors  */
/* NOTE: this defines all loci even those set as missing in rec[].mbit               */
/*************************************************************************************/
int model;
int nloci;    /* number of loci */
int anim;     /* number of genotype samples */
int rBlocks;  /* number of blocks with recaps */
int compG;    /* composite genotypes */
double total_corr_recaps; /* total corrected recaps from reference data */
{
   int n,n1,n2,i,k,m;
   int ia,ib,iaa,ibb;
   int recaps;                 /* count of number of recaps found in data to right of LLRv */
   int Amin, Bmin, Amax, Bmax;
   int blockpos[BLOCK];

   /* initialise */
   for(i=0; i < NOL; i++)
   for(k=0; k < NOA; k++)
   loc[i].count[k]=0;

   for(i=0; i < BLOCK; i++)   blockpos[i]=0;

   /* Stage 1: simulate animal samples without recaptures */
   sim_genotypes(model,nloci,anim,0,1); /* 0 no recaptures simulated in data                           */
                                      /* 1 store in animal_sim[]                                     */

   /* Stage 2: add recptures with genotype errors within blocks */
   if((model == 1)||(model == 2))
   { /* analysis using base pair data */
   }
   else
   {
      rBlocks=1;

      simBlock[0].ia=0;
      if(H_value) simBlock[0].ib=1;
      else        simBlock[0].ib=0;

      #if USE_R_VALUE
        /* simulate s.d. using R_value */
        simBlock[0].corR=R_value;
      #else
        /* simulate s.d. using estimated recapture from reference data */
        simBlock[0].corR=total_corr_recaps;
      #endif

   }


   for(m=0; m < rBlocks; m++)
   {
      ia=simBlock[m].ia;
      ib=simBlock[m].ib;
      iaa=ia+1;
      ibb=ib+1;
      Amin=nBlock[ia];          /* first position in block A */
      Bmin=nBlock[ib];          /* first position in block B */
      Amax=nBlock[iaa];         /* first position in block after A*/
      Bmax=nBlock[ibb];         /* first position in block after B */


      /* choose random recaps from decimal value of corR */
      if((simBlock[m].corR-(int)(simBlock[m].corR))  > drand() )
      {
         recaps=(int)(simBlock[m].corR)+1;      /* round up */
         simBlock[m].up++;                      /* count how many times rounded up */
      }
      else
      {
         recaps=(int)(simBlock[m].corR);      /* round down */
      }

      if(recaps < 0.0)
      {
            if(rBlocks == 1)
            {
              fprintf(fp7,"    No matches found:   Recap estimate is %d parent-offspring pairs \n",recaps);
              fprintf(stdout,"    No matches found:   Recap estimate is %d parent-offspring pairs \n",recaps);
            }
            else
            {
              fprintf(fp7,"    No matches found:   Recap estimate is %d parent-offspring pairs in populations %d (%s) and %d (%s)\n",recaps,iaa,b_name[ia],ibb,b_name[ib]);
              fprintf(stdout,"    No matches found:   Recap estimate is %d parent-offspring pairs in populations %d (%s) and %d (%s)\n",recaps,iaa,b_name[ia],ibb,b_name[ib]);
            }
            return(-1);
      }
      else
      {
#if VERBOSE
         if(rBlocks == 1)
         {
           fprintf(fp7,"  (Simulating %3d recaptures) %5.2f",recaps,simBlock[m].corR);
           fprintf(stdout,"  (Simulating %3d recaptures) %5.2f",recaps,simBlock[m].corR);
         }
         else
         {
           fprintf(fp7,"\n             (Simulating %3d recaptures in populations %2d '%s' and %d '%s')",recaps,iaa,b_name[ia],ibb,b_name[ib]);
           fprintf(stdout,"\n             (Simulating %3d recaptures in populations %2d '%s' and %d '%s')",recaps,iaa,b_name[ia],ibb,b_name[ib]);
         }
#endif

         for(n=0; n < recaps; n++) /* for n recapture animals within block m */
         {
            n1=Amin+blockpos[ia]++;

            if(((i_option[0])&&(!i_option[1]))||(model == 2 )) /* alleles read from input file */
            {
               n2=Bmin+blockpos[ib]++; /* position of match with n1 */
            }
            else          /* allele frequencies read from input file */
            {
               if(!H_value)   n2=Amax-recaps+n;                     /* position of match with n1 */
               else           n2=Bmin+blockpos[ib]++;  /* position of match with n1 */
            }

            /* try not to use same sample as duplicate in different blocks */
            /* if this cannot be avoided reset blockpos                    */
            if(blockpos[ia] > (Amax-Amin))
            {
               fprintf(fp7,"\n\nsim_individual_match_genotypes:\n");
               fprintf(fp7,"\nWarning 1: high density of recaptures in blocks - may have overwritten some pair matches blockpos[simBlock[m].ia]=%d  Amin=%d  Amax=%d\n",blockpos[ia],Amin, Amax);
               fprintf(fp7,"Overwriting pair matches may cause extra matches in diagonal block\n");   fflush(stdout);
               fprintf(fp7,"eg This could occur if R_valueS in simulation is greater than BLOCK size\n");  fflush(stdout);
               fprintf(fp7,"BLOCK number=%d  blockpos[simBlock[m].ia]=%d blockpos[simBlock[m].ib]=%d \n",rBlocks,blockpos[ia],blockpos[ib]);  fflush(stdout);

               fprintf(stdout,"\n\nsim_individual_match_genotypes:\n");
               fprintf(stdout,"\nWarning 1: high density of recaptures in blocks - may have overwritten some pair matches blockpos[simBlock[m].ia]=%d  Amin=%d  Amax=%d\n",blockpos[ia],Amin, Amax);
               fprintf(stdout,"Overwriting pair matches may cause extra matches in diagonal block\n");   fflush(stdout);
               fprintf(stdout,"eg This could occur if R_valueS in simulation is greater than BLOCK size\n");  fflush(stdout);
               fprintf(stdout,"BLOCK number=%d  blockpos[simBlock[m].ia]=%d blockpos[simBlock[m].ib]=%d \n",rBlocks,blockpos[ia],blockpos[ib]);  fflush(stdout);
               exit(1);
            }

            if((i_option[0])&&(!i_option[1])) /* alleles read from input file */
            {
               if(blockpos[ib] > (Bmax-Bmin))
               {
                  fprintf(fp7,"\nWarning 2: high density of recaps in blocks - may have overwritten some pair matches blockposA[simBlock[m].ib]=%d  Bmin=%d  Bmax=%d\n",blockpos[ia],Bmin, Bmax);
                  fprintf(fp7,"Overwriting pair matches may cause extra matches in diagonal block\n");  fflush(stdout);
                  fprintf(fp7,"eg This could occur if R_valueS in simulation is greater than BLOCK size\n");  fflush(stdout);
                  fprintf(fp7,"BLOCK number=%d  blockpos[simBlock[m].ia]=%d blockpos[simBlock[m].ib]=%d \n",rBlocks,blockpos[ia],blockpos[ib]);  fflush(stdout);

                  fprintf(stdout,"\nWarning 2: high density of recaps in blocks - may have overwritten some pair matches blockposA[simBlock[m].ib]=%d  Bmin=%d  Bmax=%d\n",blockpos[ia],Bmin, Bmax);
                  fprintf(stdout,"Overwriting pair matches may cause extra matches in diagonal block\n");  fflush(stdout);
                  fprintf(stdout,"eg This could occur if R_valueS in simulation is greater than BLOCK size\n");  fflush(stdout);
                  fprintf(stdout,"BLOCK number=%d  blockpos[simBlock[m].ia]=%d blockpos[simBlock[m].ib]=%d \n",rBlocks,blockpos[ia],blockpos[ib]);  fflush(stdout);
                  exit(1);
               }
            }
            else
            {
               if((blockpos[ib] > (Bmax-Bmin)) && (H_value) )
               {
                  fprintf(fp7,"\nWarning 3: high density of recaps in blocks - may have overwritten some pair matches blockposA[simBlock[m].ib]=%d  Bmin=%d  Bmax=%d\n",blockpos[ia],Bmin, Bmax);
                  fprintf(fp7,"Overwriting pair matches may cause extra matches in diagonal block\n");  fflush(stdout);
                  fprintf(fp7,"eg This could occur if R_valueS in simulation is greater than BLOCK size\n");  fflush(stdout);
                  fprintf(fp7,"BLOCK number=%d  blockpos[simBlock[m].ia]=%d blockpos[simBlock[m].ib]=%d \n",rBlocks,blockpos[ia],blockpos[ib]);  fflush(stdout);

                  fprintf(stdout,"\nWarning 3: high density of recaps in blocks - may have overwritten some pair matches blockposA[simBlock[m].ib]=%d  Bmin=%d  Bmax=%d\n",blockpos[ia],Bmin, Bmax);
                  fprintf(stdout,"Overwriting pair matches may cause extra matches in diagonal block\n");  fflush(stdout);
                  fprintf(stdout,"eg This could occur if R_valueS in simulation is greater than BLOCK size\n");  fflush(stdout);
                  fprintf(stdout,"BLOCK number=%d  blockpos[simBlock[m].ia]=%d blockpos[simBlock[m].ib]=%d \n",rBlocks,blockpos[ia],blockpos[ib]);  fflush(stdout);
                  exit(1);
               }
            }
            allocate_match(nloci,n1,n2);
         }
      }
   }

   /* now add genotype errors to data */
   for(i=0; i< anim;i++)
   add_loci_errors_n(nloci,1,1,E_value/10000.0,i);   /* 1 empiracle allele data used            */
                                                  /* 1 add error to animal_sim[] data */

   return(1); /* 1= more than 1 recap simulated */
}

