/*************************************************************************************/
double transition_prob(o1,o2,s1,s2,fo1,fo2)
/* coded by: Michael Macbeth       revision - 5/10/2009     */
/* Transitional probability of offspring given parent (i.e. parent = either mother of father ) */
/* returns transition probability of offspring given sire or dam */
/* for a dam *Ts = 'T(go|gm)' in Table 1 of Marshall Molecular Ecology (1998) 7, 639655  */
/*************************************************************************************/
int o1;         /* allele 1 in offspring */
int o2;         /* allele 2 in offspring */
int s1;         /* allele 1 in parent (eg sire) */
int s2;         /* allele 2 in parent (eg sire) */
double fo1,fo2; /* gene frequencies */
                /* fo1 = 'b' in Table 1 of Marshall Molecular Ecology (1998) 7, 639655  */
                /* fo2 = 'c' in Table 1 of Marshall Molecular Ecology (1998) 7, 639655  */
{
   /* ,fs1,fs2,fd1,fd2;  */
   double ts;  /* transition probability of offspring given sire */


                                                 /* xx both alleles are the same  within locus but unknown type */
                                                 /* xy allels       are different within locus but unknown type */
                                                 /* Z =(!B & !X )                                               */

                                                /* Genotypes of:            offspring    sire or dam                 */
                                                                           /******************************************/
   if(o1 ==  o2 )                                                          /* BB                                     */
   {
        if(s1 == s2)                                                       /* BB         xx                          */
        {
              if(o1 == s1)   {   ts=fo1;        }                          /* BB         BB                          */
              else           {   ts=0.0;        }                          /* BB         XX                          */
        }
        else                                                               /* BB         xy                          */
        {
           if( (s1 == o2)||
               (s2 == o2))   {   ts=fo1/2.0;   }                           /* BB         BX|XB                       */
           else              {   ts=0.0;       }                           /* BB         ZX|XZ                       */
        }
   }
   else /* o1 != o2 */                                                     /* BC       BY|YB|BB|CB|CY|YC|YY|CC|BC|YX */
   {
      if(o1 == s1)                                                         /* BC       BY|BB|BC                      */
      {
         if(s1 == s2)         {    ts=fo2;     }                           /* BC       BB                            */
         else                                                              /* BC       BY|BC                         */
         {
            if(o2 == s2)      {  ts=(fo1+fo2)/2.0; }                       /* BC       BC                            */
            else              {  ts=fo2/2.0;       }                       /* BC       BY                            */
         }
      }
      else  /* o1 ! s1 */                                                  /* BC       YB|CB|CY|YC|YY|CC|YX          */
      if(o1 == s2)                                                         /* BC       YB|CB                         */
      {
         if(o2 == s1)         { ts=(fo1+fo2)/2.0;  }                       /* BC       CB                            */
         else                 {  ts=fo2/2.0;       }                       /* BC       YB                            */
      }
      else  /* o1 ! s2 */                                                  /* BC       CY|YC|YY|CC|YX                */
      {
         if(o2 == s1)                                                      /* BC       CY|CC                         */
         {
            if(s1 == s2)      {  ts=fo1;          }                        /* BC       CC                            */
            else              {  ts=fo1/2.0;      }                        /* BC       CY                            */
         }
         else  /* o2 ! s2 */                                               /* BC       YC|YY|YX                      */
         {  
            if(s1 == s2 )     { ts=0.0;           }                        /* BC       YY                            */
            else                                                           /* BC       YC|YX                         */
            if(o2 == s2)      {  ts=fo1/2.0;      }                        /* BC       YC                            */
            else              {  ts=0.0;          }                        /* BC       YX                            */
         }
      }
   }
   return(ts);
}
